package com.smppw.analysis.application.service.position;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.NumberUtil;
import com.smppw.analysis.application.dto.position.*;
import com.smppw.analysis.domain.dao.PubliclyFundPositionDao;
import com.smppw.analysis.domain.dataobject.PubliclyFundHolderInfoDO;
import com.smppw.analysis.domain.dto.position.AssetCategoryEnum;
import com.smppw.analysis.domain.dto.position.MarketValueRatio;
import com.smppw.analysis.domain.dto.position.PositionConstants;
import com.smppw.analysis.domain.dto.position.RefMarketValueRatio;
import com.smppw.analysis.domain.dto.position.synthesize.*;
import com.smppw.analysis.domain.manager.position.BizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerFactory;
import com.smppw.common.pojo.ValueLabelVO;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

import static com.smppw.analysis.domain.manager.position.BizHandlerConstants.*;

/**
 * @author wangzaijun
 * @date 2023/6/6 14:40
 * @description 基金持仓分析-综合tab
 */
@Component
public class SynthesizePositionAnalysis {
    private static final Map<String, String> ASSET_TYPE_MAPPER = MapUtil.newHashMap(true);

    static {
        for (AssetCategoryEnum value : AssetCategoryEnum.values()) {
            ASSET_TYPE_MAPPER.put(value.name(), value.getName());
        }
        ASSET_TYPE_MAPPER.put(PositionConstants.ASSET_SALE, PositionConstants.ASSET_SALE_DESC);
        ASSET_TYPE_MAPPER.put(PositionConstants.ASSET_RETURN_SALE, PositionConstants.ASSET_RETURN_SALE_DESC);
        ASSET_TYPE_MAPPER.put(PositionConstants.ASSET_MONEY, PositionConstants.ASSET_MONEY_DESC);
        ASSET_TYPE_MAPPER.put(PositionConstants.ASSET_OTHER, PositionConstants.ASSET_OTHER_DESC);
    }

    private final BizHandlerFactory factor;
    private final PubliclyFundPositionDao fundPositionBaseService;

    public SynthesizePositionAnalysis(BizHandlerFactory factor, PubliclyFundPositionDao fundPositionBaseService) {
        this.factor = factor;
        this.fundPositionBaseService = fundPositionBaseService;
    }

    public Map<String, Object> getAssetAllocation(AssetAllocationReq req) {
        AssetAllocationParams params = req.convert();
        BizHandler<AssetAllocationParams, List<AssetAllocationVO>> bizHandler = this.factor.getBizHandlerInstance(ASSET_ALLOCATION);
        List<AssetAllocationVO> dataList = bizHandler.bizHandle(params);
        if (CollUtil.isEmpty(dataList)) {
            return MapUtil.empty();
        }
        List<ValueLabelVO> mapping = ListUtil.list(true);
        Map<String, Object> dataset = MapUtil.newHashMap();
        ASSET_TYPE_MAPPER.forEach((k, v) -> {
            List<MarketValueRatio> collect = dataList.stream().map(e -> {
                RefMarketValueRatio mvr = e.getAsset().stream()
                        .filter(o -> k.equals(o.getRef().getValue())).findFirst().orElse(null);
                MarketValueRatio data = new MarketValueRatio();
                data.setDate(e.getDate());
                data.setMarketValue(Optional.ofNullable(mvr).map(RefMarketValueRatio::getMarketValue).orElse(BigDecimal.ZERO));
                data.setRatio(Optional.ofNullable(mvr).map(RefMarketValueRatio::getRatio).orElse(BigDecimal.ZERO));
                return data;
            }).collect(Collectors.toList());
            dataset.put(k, collect);
            mapping.add(new ValueLabelVO(k, v));
        });
        return MapUtil.<String, Object>builder().put("dataset", dataset).put("productNameMapping", mapping).put("table", dataList).build();
    }

    public List<LeverageChangeVO> getLeverageChange(LeverageChangeReq req) {
        LeverageChangeParams params = req.convert();
        BizHandler<LeverageChangeParams, List<LeverageChangeVO>> bizHandler = this.factor.getBizHandlerInstance(LEVERAGE_CHANGE);
        List<LeverageChangeVO> dataList = bizHandler.bizHandle(params);
        if (CollUtil.isEmpty(dataList)) {
            return ListUtil.empty();
        }
        return dataList;
    }

    public PositionInfoVO getPositionParams(PositionInfoReq req) {
        PositionInfoParams params = req.convert();
        BizHandler<PositionInfoParams, PositionInfoVO> bizHandler = this.factor.getBizHandlerInstance(POSITION_PARAMS);
        return bizHandler.bizHandle(params);
    }

    public List<PositionListVO> getPosition(PositionListReq req) {
        PositionListParams params = req.convert();
        BizHandler<PositionListParams, List<PositionListVO>> bizHandler = this.factor.getBizHandlerInstance(POSITION_LIST);
        List<PositionListVO> dataList = bizHandler.bizHandle(params);
        if (CollUtil.isEmpty(dataList)) {
            return ListUtil.empty();
        }
        return dataList;
    }

    public List<HolderInfoVO> getHolderInfo(HolderInfoReq req) {
        HolderInfoParams params = req.convert();
        List<HolderInfoVO> resultList = ListUtil.list(true);
        List<PubliclyFundHolderInfoDO> dataList = this.fundPositionBaseService.mfHolderInfoList(params.getFundId(), params.getStartDate(), params.getEndDate());
        for (PubliclyFundHolderInfoDO data : dataList) {
            HolderInfoVO vo = new HolderInfoVO();
            vo.setDate(DateUtil.formatDate(data.getEndDate()));
            vo.setAverageHoldShares(Optional.ofNullable(data.getAverageHoldShares()).map(BigDecimal::toPlainString).orElse(null));
            vo.setEtfFeederHoldRatio(Optional.ofNullable(data.getEtfFeederHoldRatio()).map(BigDecimal::toPlainString).orElse(null));
            vo.setHolderAccountNumber(Optional.ofNullable(data.getHolderAccountNumber()).map(Object::toString).orElse(null));
            vo.setIndividualHoldRatio(Optional.ofNullable(data.getIndividualHoldRatio()).map(BigDecimal::toPlainString).orElse(null));
            vo.setInstitutionHoldRatio(Optional.ofNullable(data.getInstitutionHoldRatio()).map(BigDecimal::toPlainString).orElse(null));
            vo.setProfessionalHoldRatio(Optional.ofNullable(data.getProfessionalHoldRatio()).map(BigDecimal::toPlainString).orElse(null));
            vo.setTop10HolderAmount(Optional.ofNullable(data.getTop10HolderAmount()).map(BigDecimal::toPlainString).orElse(null));
            vo.setTop10HoldersProportion(Optional.ofNullable(data.getTop10HoldersProportion()).map(BigDecimal::toPlainString).orElse(null));
            // 计算总份额=机构持有份额+个人持有份额+未明确投资者持有份额
            BigDecimal total = NumberUtil.add(ListUtil.of(data.getInstitutionHoldShares(), data.getIndividualHoldShares(), data.getUndefinedHoldShares())
                    .stream().filter(Objects::nonNull).toArray(BigDecimal[]::new));
            vo.setTotal(total.toPlainString());
            resultList.add(vo);
        }
        // 日期倒序
        resultList.sort((o1, o2) -> o2.getDate().compareTo(o1.getDate()));
        return resultList;
    }
}
