package com.smppw.analysis.client;

import com.smppw.analysis.application.dto.position.*;
import com.smppw.analysis.application.service.position.FundFuturesOptionService;
import com.smppw.analysis.application.service.position.FuturePositionAnalysis;
import com.smppw.analysis.application.service.position.StockPositionAnalysis;
import com.smppw.analysis.application.service.position.SynthesizePositionAnalysis;
import com.smppw.analysis.domain.dto.position.bond.BondSortAllocationParam;
import com.smppw.analysis.domain.dto.position.stock.ChangeNumberVO;
import com.smppw.analysis.domain.dto.position.stock.MajorChangeVO;
import com.smppw.analysis.domain.dto.position.synthesize.HolderInfoVO;
import com.smppw.analysis.domain.dto.position.synthesize.LeverageChangeVO;
import com.smppw.analysis.domain.dto.position.synthesize.PositionInfoVO;
import com.smppw.analysis.domain.dto.position.synthesize.PositionListVO;
import com.smppw.analysis.domain.manager.position.bond.BondPositionService;
import com.smppw.common.pojo.ResultVo;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * @author wangzaijun
 * @date 2023/6/6 11:23
 * @description 基金 持仓分析，包括公募、私募基金，区分综合、股票、债券和期货tab组件
 */
@RestController
@RequestMapping("/v1/api/position")
public class FundPositionApi {
    //        private final BondPositionAnalysis bond;
    private final StockPositionAnalysis stock;
    private final FuturePositionAnalysis future;
    private final SynthesizePositionAnalysis synthesize;
    private final BondPositionService bondPositionService;
    private final FundFuturesOptionService fundFuturesOptionService;

    public FundPositionApi(StockPositionAnalysis stock, FuturePositionAnalysis future,
                           SynthesizePositionAnalysis synthesize, BondPositionService bondPositionService,
                           FundFuturesOptionService fundFuturesOptionService) {
        this.stock = stock;
        this.future = future;
        this.synthesize = synthesize;
        this.bondPositionService = bondPositionService;
        this.fundFuturesOptionService = fundFuturesOptionService;
    }

    /**
     * 公募私募，综合-资产大类配置
     *
     * @param params /
     * @return /
     */
    @GetMapping("asset-allocation")
    public ResultVo<Map<String, Object>> getAssetAllocation(AssetAllocationReq params) {
        return ResultVo.ok(this.synthesize.getAssetAllocation(params));
    }

    /**
     * 公募私募，综合-杠杆变化
     *
     * @param params /
     * @return /
     */
    @GetMapping("leverage-change")
    public ResultVo<List<LeverageChangeVO>> getLeverageChange(LeverageChangeReq params) {
        return ResultVo.ok(this.synthesize.getLeverageChange(params));
    }

    /**
     * 公募私募，综合-持仓列表的请求参数
     *
     * @param params /
     * @return /
     */
    @GetMapping("position-param")
    public ResultVo<PositionInfoVO> getPositionParams(PositionInfoReq params) {
        return ResultVo.ok(this.synthesize.getPositionParams(params));
    }

    /**
     * 公募私募，综合-持仓列表
     *
     * @param params /
     * @return /
     */
    @GetMapping("position-list")
    public ResultVo<List<PositionListVO>> getPosition(PositionListReq params) {
        return ResultVo.ok(this.synthesize.getPosition(params));
    }

    /**
     * 公募，综合-持有人结构
     *
     * @param params /
     * @return /
     */
    @GetMapping("holder-info")
    public ResultVo<List<HolderInfoVO>> getHolderInfo(HolderInfoReq params) {
        return ResultVo.ok(this.synthesize.getHolderInfo(params));
    }

    /**
     * 公募，股票-持仓股票重大变动
     *
     * @param params /
     * @return /
     */
    @GetMapping("stock/major-change")
    public ResultVo<List<MajorChangeVO>> getMajorChange(MajorChangeReq params) {
        return ResultVo.ok(this.stock.getMajorChangeList(params));
    }

    /**
     * 公募私募，股票-集中度
     *
     * @param params /
     * @return /
     */
    @GetMapping("stock/crn")
    public ResultVo<Map<String, Object>> getStockConcentration(ConcentrationReq params) {
        return ResultVo.ok(this.stock.getConcentration(params));
    }

    /**
     * 公募私募，股票-持股数量变动时序
     *
     * @param params /
     * @return /
     */
    @GetMapping("stock/change-num")
    public ResultVo<List<ChangeNumberVO>> getStockChangeNumber(ChangeNumberReq params) {
        return ResultVo.ok(this.stock.getChangeNumber(params));
    }

    /**
     * 公募私募，股票-行业配置
     *
     * @param params /
     * @return /
     */
    @GetMapping("stock/industry-allocation")
    public ResultVo<Map<String, Object>> getStockIndustryAllocation(StockAllocationReq params) {
        return ResultVo.ok(this.stock.getIndustryAllocation(params));
    }

    /**
     * 公募私募，股票-行业配置偏好时序
     *
     * @param params /
     * @return /
     */
    @GetMapping("stock/industry-allocation-preference")
    public ResultVo<Map<String, Object>> getStockIndustryAllocationPreference(StockAllocationReq params) {
        return ResultVo.ok(this.stock.getIndustryAllocationPreference(params));
    }

    /**
     * 公募私募，股票-风格配置
     *
     * @param params /
     * @return /
     */
    @GetMapping("stock/style-allocation")
    public ResultVo<Map<String, Object>> getStockStyleAllocation(StockAllocationReq params) {
        return ResultVo.ok(this.stock.getStyleAllocation(params));
    }

    /**
     * 公募私募，股票-流动性分析
     *
     * @param params /
     * @return /
     */
    @GetMapping("stock/liquidity-allocation")
    public ResultVo<Map<String, Object>> getStockLiquidityAllocation(StockAllocationReq params) {
        return ResultVo.ok(this.stock.getLiquidityAllocation(params));
    }

    /**
     * 公募私募，股票-业绩归因
     *
     * @param params /
     * @return /
     */
    @GetMapping("stock/performance-attribution")
    public ResultVo<Map<String, Object>> getStockPerformanceAttribution(StockPerformanceAttributionReq params) {
        return ResultVo.ok(this.stock.getPerformanceAttribution(params));
    }

    /**
     * 公募私募，股票-Barra敏感度分析
     *
     * @param params /
     * @return /
     */
    @GetMapping("stock/barra-sensitivity")
    public ResultVo<Map<String, Object>> getStockBarraSensitivity(BarraSensitivityReq params) {
        return ResultVo.ok(this.stock.getBarraSensitivity(params));
    }

    /**
     * 债券分类配置及明细  公募私募
     *
     * @param param /
     * @return /
     */
    @GetMapping("bond/sort-allocation")
    public ResultVo<Map<String, Object>> getBondSortAllocation(BondSortAllocationParam param) {
        return ResultVo.ok(bondPositionService.getBondSortAllocation(param));
    }

    /**
     * 债券集中度
     *
     * @param param /
     * @return /
     */
    @GetMapping("bond/crn")
    public ResultVo<Map<String, Object>> getBondConcentration(BondSortAllocationParam param) {
        return ResultVo.ok(bondPositionService.getBondConcentration(param));
    }

    /**
     * 债券信用评级
     *
     * @param param /
     * @return /
     */
    @GetMapping("/bond/credit-grading")
    public ResultVo<Map<String, Object>> getBondCreditGrading(BondSortAllocationParam param) {
        return ResultVo.ok(bondPositionService.getBondCreditGrading(param));
    }

    /**
     * @param fundId     基金ID
     * @param optionType 板块类别
     * @param shType     投机/套期 1-Speculation,2-Hedging
     * @return
     */
    @RequestMapping(value = "/fund-futures-option", method = RequestMethod.GET)
    public ResultVo<Map<String, Object>> fundFuturesOption(@RequestParam("userId") Integer userId, @RequestParam("fundId") String fundId, @RequestParam("optionType") Integer optionType, @RequestParam(value = "shType", defaultValue = "1") Integer shType,
                                                           @RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate) {
        return fundFuturesOptionService.fundFuturesOption(userId, fundId, optionType, shType, startDate, endDate);
    }


    /**
     * @param fundId     基金ID
     * @param optionType 板块类别
     * @param shType     投机/套期 1-Speculation,2-Hedging
     * @return
     */
    @RequestMapping(value = "/fund-futures-option-breed", method = RequestMethod.GET)
    public ResultVo<Map<String, Object>> fundFuturesOptionForBreed(@RequestParam("userId") Integer userId, @RequestParam("fundId") String fundId, @RequestParam(value = "optionType", required = false) Integer optionType, @RequestParam(value = "shType", defaultValue = "1") Integer shType,
                                                                   @RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate) {
        return fundFuturesOptionService.fundFuturesOptionForBreed(userId, fundId, optionType, shType, startDate, endDate);
    }

    /**
     * 板块持仓分析
     *
     * @param fundId
     * @param optionType
     * @param shType
     * @return
     */
    @RequestMapping(value = "/fund-position-analysis", method = RequestMethod.GET)
    public ResultVo<Map<String, Object>> fundPositionAnalysis(@RequestParam("userId") Integer userId, @RequestParam("fundId") String fundId, @RequestParam("optionType") Integer optionType, @RequestParam(value = "shType", defaultValue = "1") Integer shType,
                                                              @RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate) {
        return fundFuturesOptionService.fundPositionAnalysis(userId, fundId, optionType, shType, startDate, endDate);
    }

    /**
     * 板块持仓分析
     *
     * @param fundId
     * @param optionType
     * @param shType
     * @return
     */
    @RequestMapping(value = "/fund-position-analysis-line", method = RequestMethod.GET)
    public ResultVo<Map<String, Object>> fundPositionAnalysisLine(@RequestParam("userId") Integer userId, @RequestParam("fundId") String fundId, @RequestParam(value = "optionType", required = false, defaultValue = "0") Integer optionType, @RequestParam(value = "shType", defaultValue = "1", required = false) Integer shType,
                                                                  @RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate) {
        return fundFuturesOptionService.fundPositionAnalysisLine(userId, fundId, optionType, shType, startDate, endDate);
    }


    /**
     * 衍生品杠杆与保证金
     *
     * @param fundId
     * @return
     */
    @RequestMapping(value = "/fund-derivative-leverage-margin", method = RequestMethod.GET)
    public ResultVo<Map<String, Object>> fundDerivativeLeverageMargin(@RequestParam("userId") Integer userId, @RequestParam("fundId") String fundId, @RequestParam(value = "shType", required = false) Integer shType,
                                                                      @RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate) {
        return fundFuturesOptionService.fundDerivativeLeverageMargin(userId, fundId, shType, startDate, endDate);
    }


    /**
     * 边际风险贡献
     *
     * @param params
     * @return
     */
    @RequestMapping(value = "/fund-marginal-risk-contribution", method = RequestMethod.GET)
    public ResultVo<Map<String, Object>> getFutureRiskCont(MarginalRiskContributionReq params) {
        return ResultVo.ok(this.future.riskCont(params));
    }

    /**
     * @return
     */
    @RequestMapping(value = "/fund-futures-option-view", method = RequestMethod.GET)
    public ResultVo<Map<String, List<String>>> fundFuturesOptionView() {
        return fundFuturesOptionService.fundFuturesOptionView();
    }

//    /**
//     * 公募私募，债券-久期分析
//     *
//     * @param params /
//     * @return /
//     */
//    @GetMapping("bond/duration-analysis")
//    public ResultVo<List<DurationAnalysisVO>> getBondDurationAnalysis(DurationAnalysisParams params) {
//        return null;
//    }
//
//    /**
//     * 公募私募，债券-收益风险
//     *
//     * @param params /
//     * @return /
//     */
//    @GetMapping("bond/profit-risk")
//    public ResultVo<List<ProfitRiskVO>> getBondProfitRisk(ProfitRiskParams params) {
//        return null;
//    }
//
//    /**
//     * 公募私募，债券-业绩归因
//     *
//     * @param params /
//     * @return /
//     */
//    @GetMapping("bond/performance-attribution")
//    public ResultVo<List<BondPerformanceAttributionVO>> getBondPerformanceAttribution(BondPerformanceAttributionParams params) {
//        return null;
//    }
//
//    /**
//     * 公募私募，债券-债券分类配置及明细
//     *
//     * @param params /
//     * @return /
//     */
//    @GetMapping("bond/asset-allocation")
//    public ResultVo<List<BondAssetAllocationVO>> getBondAssetAllocation(BondAssetAllocationParams params) {
//        return null;
//    }
//
//    /**
//     * 公募私募，债券-集中度
//     *
//     * @param params /
//     * @return /
//     */
//    @GetMapping("bond/crn")
//    public ResultVo<List<ConcentrationVO>> getBondConcentration(ConcentrationParams params) {
//        return null;
//    }
//
//    @GetMapping("future/risk-cont")
//    public ResultVo<Map<String, Object>> getFutureRiskCont(MarginalRiskContributionParams params) {
//        return ResultVo.ok(this.future.riskCont(params));
//    }
}
