package com.smppw.analysis.domain.manager.performance;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.performance.Params;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.analysis.infrastructure.exception.APIException;
import com.smppw.common.cache.CaffeineLocalCache;
import com.smppw.common.pojo.enums.Frequency;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Objects;
import java.util.function.BinaryOperator;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/8/8 9:54
 * @description 抽象的业绩分析接口处理，模板方法模式
 */
public abstract class AbstractPerformance<P extends Params, R> implements Performance<P, R> {
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());
    protected final BaseInfoService baseInfoService;
    protected final BaseIndicatorServiceV2 baseIndicatorServiceV2;

    public AbstractPerformance(BaseInfoService baseInfoService, BaseIndicatorServiceV2 baseIndicatorServiceV2) {
        this.baseInfoService = baseInfoService;
        this.baseIndicatorServiceV2 = baseIndicatorServiceV2;
    }

    /**
     * 利用jdk8的流操作计算最大最小值
     *
     * @param dataList 待计算的数据序列
     * @param operator 求最大还最小值的操作函数
     * @return /
     */
    protected static Double handleMaxAndMin(List<Double> dataList, BinaryOperator<Double> operator) {
        if (dataList == null || dataList.isEmpty()) {
            return 0d;
        }
        List<Double> collect = dataList.stream().filter(Objects::nonNull).collect(Collectors.toList());
        if (collect.isEmpty()) {
            return 0d;
        }
        return collect.stream().reduce(operator).orElse(0d);
    }

    /**
     * 滚动指标 时间字段特殊处理
     *
     * @param date             时间
     * @param rollingFrequency 滚动频率
     * @return /
     */
    protected static String handleStrDate(String date, Frequency rollingFrequency) {
        if (rollingFrequency == Frequency.Weekly) {
            String yearMonthWeek = CaffeineLocalCache.getDateMonthWeek(date);
            String year = yearMonthWeek.substring(0, 4);
            String month = yearMonthWeek.substring(5, 7);
            if (month.startsWith("0")) {
                month = month.substring(1);
            }
            String week = yearMonthWeek.substring(8);
            return year + "年" + month + "月第" + week + "周";
        }
        return date;
    }

    @Override
    public R execute(P params) {
        try {
            this.checkParams(params);
            return this.bizHandle(params);
        } catch (Exception e) {
            this.logger.warn(StrUtil.format("接口请求错误！{}", e.getMessage()));
        }
        return null;
    }

    protected void checkParams(P params) {
        if (CollUtil.isEmpty(params.getRefIds())) {
            throw new APIException("标的id不能为空！");
        }
        if (CharSequenceUtil.isAllBlank(params.getStartDate(), params.getEndDate())) {
            throw new APIException("开始日期和结束日期不能为空！");
        }
    }

    protected abstract R bizHandle(P params);
}
