package com.smppw.analysis.domain.manager.performance;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.performance.BaseParams;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.common.pojo.dto.indicator.CalcMultipleSecMultipleTimeRangeIndicatorReq;
import com.smppw.common.pojo.dto.indicator.DateIntervalDto;
import com.smppw.common.pojo.enums.DateIntervalType;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.common.pojo.enums.Visibility;
import com.smppw.utils.StrategyHandleUtils;

import java.util.List;
import java.util.Map;

/**
 * @author wangzaijun
 * @date 2023/8/8 9:54
 * @description 抽象的单标的的业绩表现
 */
public abstract class AbstractSingleSecPerformance<P extends BaseParams, R> extends AbstractPerformance<P, R> {
    public AbstractSingleSecPerformance(BaseInfoService baseInfoService, BaseIndicatorServiceV2 baseIndicatorServiceV2) {
        super(baseInfoService, baseIndicatorServiceV2);
    }

    @Override
    protected R bizHandle(P params) {
        return this.bizSecHandle(params);
    }

    protected abstract R bizSecHandle(P params);

    /**
     * 从请求参数中获取所有标的并集集合
     *
     * @param params 公共请求参数
     * @return /
     */
    protected List<String> getSecIdsByParams(P params) {
        List<String> tempList = ListUtil.list(true);
        tempList.addAll(params.getRefIds());
        if (StrUtil.isNotBlank(params.getBenchmarkId())) {
            tempList.add(params.getBenchmarkId());
        }
        return params.getSecIds() == null ? tempList : CollUtil.addAllIfNotContains(tempList, params.getSecIds());
    }

    /**
     * 根据请求参数构建 DateIntervalDto 对象
     *
     * @param params           请求参数，需继承 CommonParams
     * @param dateIntervalType 时段类型
     * @param frequency        接口传递的频率，一般为滚动计算时传递的滚动频率
     * @return /
     */
    protected DateIntervalDto buildDateIntervalByParams(P params, DateIntervalType dateIntervalType, Frequency frequency) {
        if (dateIntervalType == null) {
            dateIntervalType = DateIntervalType.DefaultInterval;
        }
        if (frequency == null) {
            frequency = params.getFrequency();
        }
        return DateIntervalDto.builder()
                .id(dateIntervalType.name())
                .startDate(params.getStartDate())
                .endDate(params.getEndDate())
                .timeRange(params.getTimeRange())
                .dateIntervalType(dateIntervalType)
                .frequency(frequency).build();
    }

    protected CalcMultipleSecMultipleTimeRangeIndicatorReq buildCalcReq(P params, DateIntervalType dateIntervalType) {
        List<String> refIds = params.getRefIds();
        List<String> secIds = this.getSecIdsByParams(params);
        // 差集求指数id集合，包括基准
        List<String> indexIds = CollectionUtil.subtractToList(secIds, refIds);
        Map<String, String> benchmarkIdMap = MapUtil.newHashMap(refIds.size(), true);
        for (String refId : refIds) {
            if (StrUtil.isBlank(params.getBenchmarkId())) {
                continue;
            }
            benchmarkIdMap.put(refId, params.getBenchmarkId());
        }
        DateIntervalDto dateInterval = this.buildDateIntervalByParams(params, dateIntervalType, params.getFrequency());
        Map<String, List<DateIntervalDto>> dateIntervalMap = MapUtil.newHashMap(refIds.size(), true);
        for (String refId : refIds) {
            dateIntervalMap.put(refId, ListUtil.toList(dateInterval));
        }
        return CalcMultipleSecMultipleTimeRangeIndicatorReq.builder()
                .mainSecIdList(refIds)
                .secBenchmarkIdMap(benchmarkIdMap)
                .indexIdList(indexIds)
                .raiseType(params.getRaiseType())
                .strategy(StrategyHandleUtils.getStrategy(params.getStrategy()))
                .visibility(Visibility.Both)
                .dataFrequency(params.getFrequency())
                .secDateIntervalDtoListMap(dateIntervalMap)
                .navType(params.getNavType())
                .ifAnnualize(true)
                .calcIndexRetIndicatorValue(true)
                .ifConvertPerformanceConsistencyWord(true).build();
    }
}
