package com.smppw.analysis.domain.manager.performance.handler;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.performance.IndicatorParams;
import com.smppw.analysis.domain.manager.performance.AbstractSingleSecPerformance;
import com.smppw.analysis.domain.manager.performance.PerformanceConstants;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.common.pojo.dto.calc.IndicatorCalcIndexDataDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcPropertyDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcSecDataDto;
import com.smppw.common.pojo.dto.indicator.CalcMultipleSecMultipleTimeRangeIndicatorReq;
import com.smppw.common.pojo.enums.DateIntervalType;
import com.smppw.common.pojo.enums.Indicator;
import com.smppw.constants.Consts;
import com.smppw.core.reta.calc.PerformanceConsistency;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@Component(PerformanceConstants.INDICATOR)
public class IndicatorHandler extends AbstractSingleSecPerformance<IndicatorParams, Map<String, Object>> {
    public IndicatorHandler(BaseInfoService baseInfoService, BaseIndicatorServiceV2 baseIndicatorServiceV2) {
        super(baseInfoService, baseIndicatorServiceV2);
    }

    @Override
    protected CalcMultipleSecMultipleTimeRangeIndicatorReq buildCalcReq(IndicatorParams params, DateIntervalType dateIntervalType) {
        CalcMultipleSecMultipleTimeRangeIndicatorReq req = super.buildCalcReq(params, dateIntervalType);
        String riskOfFreeId = params.getRiskOfFreeId();
        Double riskOfFreeValue = params.getRiskOfFreeValue();
        if (StrUtil.isBlank(riskOfFreeId)) {
            riskOfFreeId = Consts.RISK_OF_FREE;
        } else if (NumberUtil.isNumber(riskOfFreeId)) {
            riskOfFreeValue = Double.parseDouble(riskOfFreeId);
            riskOfFreeId = null;
        }
        req.setRiskOfFreeId(riskOfFreeId);
        req.setRiskOfFreeValue(riskOfFreeValue);
        req.setIndicatorList(params.getIndicators());
        req.setGeoExtraindicatorList(params.getExtraIndicators());
        return req;
    }

    @Override
    protected Map<String, Object> bizSecHandle(IndicatorParams params) {
        // 1.参数处理
        List<String> refIds = params.getRefIds();
        List<String> secIds = this.getSecIdsByParams(params);
        List<String> indexIds = CollectionUtil.subtractToList(secIds, refIds);
        CalcMultipleSecMultipleTimeRangeIndicatorReq req = this.buildCalcReq(params, DateIntervalType.CustomInterval);
        // 2.指标计算
        Map<String, List<IndicatorCalcPropertyDto>> indicator = this.baseIndicatorServiceV2.calcMultipleSecMultipleTimeRangeIndicator(req);
        // 3.返回结果处理
        Map<String, Object> valuesMap = MapUtil.newHashMap(true);
        Map<String, Object> indexValuesMap = MapUtil.newHashMap(true);
        for (String refId : refIds) {
            IndicatorCalcPropertyDto dto = Optional.ofNullable(indicator.get(refId)).filter(e -> !e.isEmpty()).map(e -> e.get(0)).orElse(null);
            // 标的指标
            Map<String, String> indicatorValues = Optional.ofNullable(dto).map(IndicatorCalcPropertyDto::getSecData).map(IndicatorCalcSecDataDto::getIndicatorValueMap).orElse(MapUtil.empty());
            // 标的几何指标
            Map<String, String> geoIndicatorValues = Optional.ofNullable(dto).map(IndicatorCalcPropertyDto::getSecData).map(IndicatorCalcSecDataDto::getExtraGeoIndicatorValueMap).orElse(MapUtil.empty());
            // 指数指标
            Map<String, Map<String, String>> indexIndicatorValuesMap = Optional.ofNullable(dto).map(IndicatorCalcPropertyDto::getIndexData).map(IndicatorCalcIndexDataDto::getIndexIndicatorValueMap).orElse(MapUtil.empty());
            // 指数几何指标
            Map<String, Map<String, String>> indexGeoIndicatorValuesMap = Optional.ofNullable(dto).map(IndicatorCalcPropertyDto::getIndexData).map(IndicatorCalcIndexDataDto::getIndexGeoExtraIndicatorValueMap).orElse(MapUtil.empty());
            // 标的和指数分别处理最大回撤指标，并合并指标结果到一个map
            Map<String, String> values = this.handleMaxDrawdown(indicatorValues, refId, null, false);
            Map<String, String> geoValues = this.handleMaxDrawdown(geoIndicatorValues, refId, null, true);
            Map<String, String> unionValues = MapUtil.<String, String>builder().putAll(values).putAll(geoValues).build();
            valuesMap.put(refId, unionValues);
            for (String indexId : indexIds) {
                Map<String, String> indexValues = this.handleMaxDrawdown(indexIndicatorValuesMap.get(indexId), indexId, null, false);
                Map<String, String> indexGeoValues = this.handleMaxDrawdown(indexGeoIndicatorValuesMap.get(indexId), indexId, params.getBenchmarkId(), true);
                Map<String, String> unionIndexValues = MapUtil.<String, String>builder().putAll(indexValues).putAll(indexGeoValues).build();
                indexValuesMap.put(indexId, unionIndexValues);
            }
        }
        // 返回结果对象构建
        Map<String, String> productNameMapping = this.baseInfoService.querySecName(secIds);
        Map<String, Object> dataset = MapUtil.builder(valuesMap).putAll(indexValuesMap).build();
        return MapUtil.<String, Object>builder().put("dataset", dataset).put("productNameMapping", productNameMapping).build();
    }

    protected Map<String, String> handleMaxDrawdown(Map<String, String> indicatorValues, String secId, String benchmarkId, boolean geo) {
        Map<String, String> result = MapUtil.newHashMap();
        String maxDrawdown = indicatorValues.get(Indicator.MaxDrawdown.name());
        String consistency = indicatorValues.get(Indicator.PerformanceConsistency.name());
        if (StrUtil.isNotBlank(maxDrawdown) && new BigDecimal(maxDrawdown).compareTo(BigDecimal.ZERO) > 0) {
            if (StrUtil.isBlank(indicatorValues.get(Indicator.MaxDrawdownRecureDate.name()))) {
                indicatorValues.put(Indicator.MaxDrawdownRecureDate.name(), "最大回撤未修复");
            }
            if (StrUtil.isBlank(indicatorValues.get(Indicator.MaxDrawdownRecureIntervalDays.name()))) {
                indicatorValues.put(Indicator.MaxDrawdownRecureIntervalDays.name(), "最大回撤未修复");
            }
            if (StrUtil.isBlank(indicatorValues.get(Indicator.MaxDrawdownRecureInterval.name()))) {
                indicatorValues.put(Indicator.MaxDrawdownRecureInterval.name(), "最大回撤未修复");
            }
        }
        if (StrUtil.isEmpty(consistency)) {
            indicatorValues.put(Indicator.PerformanceConsistency.name(), PerformanceConsistency.convert2Word(null));
        } else {
            indicatorValues.put(Indicator.PerformanceConsistency.name(), PerformanceConsistency.convert2Word(Double.parseDouble(consistency)));
        }
        if (geo) {
            indicatorValues.forEach((k, v) -> result.put("geoExcess" + k, secId != null && secId.equals(benchmarkId) ? null : v));
        } else {
            result.putAll(indicatorValues);
        }
        return result;
    }
}
