package com.smppw.analysis.domain.manager.performance.handler;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.lang.Pair;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.performance.HistoryRetTableSingleResp;
import com.smppw.analysis.domain.dto.performance.RevenueParams;
import com.smppw.analysis.domain.manager.performance.AbstractSingleSecPerformance;
import com.smppw.analysis.domain.manager.performance.HistoryRetTableUtil;
import com.smppw.analysis.domain.manager.performance.PerformanceConstants;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.analysis.infrastructure.exception.APIException;
import com.smppw.common.pojo.dto.calc.*;
import com.smppw.common.pojo.dto.indicator.CalcMultipleSecMultipleTimeRangeIndicatorReq;
import com.smppw.common.pojo.dto.indicator.DateIntervalDto;
import com.smppw.common.pojo.enums.DateIntervalType;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.common.pojo.enums.Indicator;
import com.smppw.common.pojo.enums.Visibility;
import com.smppw.constants.Consts;
import com.smppw.utils.DataUtil;
import com.smppw.utils.RangeUtils;
import com.smppw.utils.StrategyHandleUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Component(PerformanceConstants.REVENUE)
public class RevenueHandler extends AbstractSingleSecPerformance<RevenueParams, Map<String, Object>> {
    private static final String STATISTICS = "1";
    private static final String DISTRIBUTION = "2";
    private static final List<String> TYPE_LIST = ListUtil.of(STATISTICS, DISTRIBUTION);

    public RevenueHandler(BaseInfoService baseInfoService, BaseIndicatorServiceV2 baseIndicatorServiceV2) {
        super(baseInfoService, baseIndicatorServiceV2);
    }

    @Override
    protected void checkParams(RevenueParams params) {
        super.checkParams(params);
        if (StrUtil.isBlank(params.getRevenuType()) || !TYPE_LIST.contains(params.getRevenuType())) {
            params.setRevenuType(STATISTICS);
        }
        List<Frequency> frequencies = ListUtil.toList(Frequency.Weekly, Frequency.Monthly, Frequency.Quarterly, Frequency.Annually);
        if (params.getRollingFrequency() == null || !frequencies.contains(params.getRollingFrequency())) {
            logger.warn(String.format("当前滚动频率为：%s，设置为默认滚动频率：%s", params.getRollingFrequency(), Frequency.Monthly));
            params.setRollingFrequency(Frequency.Monthly);
        }
        //支持调整组距 1~20%
        if (DISTRIBUTION.equals(params.getRevenuType())) {
            if (!NumberUtil.isNumber(params.getSpace())) {
                throw new APIException("组距必须是数字");
            }
            BigDecimal space = NumberUtil.toBigDecimal(params.getSpace());
            if (!NumberUtil.isIn(space, BigDecimal.ONE, NumberUtil.toBigDecimal("20"))) {
                throw new APIException("组距必须是大于1小于20的数");
            }
        }
    }

    @Override
    protected CalcMultipleSecMultipleTimeRangeIndicatorReq buildCalcReq(RevenueParams params, DateIntervalType dateIntervalType) {
        Frequency rollingFrequency = params.getRollingFrequency();
        List<String> refIds = params.getRefIds();
        String startDate = params.getStartDate();
        String endDate = params.getEndDate();
        List<DateIntervalDto> dateIntervalDtoList = new ArrayList<>();
        DateIntervalDto dateIntervalDto = new DateIntervalDto();
        dateIntervalDto.setDateIntervalType(DateIntervalType.DefaultRolling);
        dateIntervalDto.setStartDate(startDate);
        dateIntervalDto.setEndDate(endDate);
        dateIntervalDto.setFrequency(rollingFrequency);
        dateIntervalDtoList.add(dateIntervalDto);
        if (Frequency.Monthly == rollingFrequency || Frequency.Quarterly == rollingFrequency) {
            DateIntervalDto yearDateIntervalDto = new DateIntervalDto();
            yearDateIntervalDto.setDateIntervalType(DateIntervalType.DefaultRolling);
            yearDateIntervalDto.setStartDate(startDate);
            yearDateIntervalDto.setEndDate(endDate);
            yearDateIntervalDto.setFrequency(Frequency.Annually);
            yearDateIntervalDto.setId(HistoryRetTableUtil.YEAR_ROLL_DEFAULT_ID);
            dateIntervalDtoList.add(yearDateIntervalDto);
        } else if (Frequency.Weekly == rollingFrequency) {
            DateIntervalDto monthDateIntervalDto = new DateIntervalDto();
            monthDateIntervalDto.setDateIntervalType(DateIntervalType.DefaultRolling);
            monthDateIntervalDto.setStartDate(startDate);
            monthDateIntervalDto.setEndDate(endDate);
            monthDateIntervalDto.setFrequency(Frequency.Monthly);
            monthDateIntervalDto.setId(HistoryRetTableUtil.MONTH_ROLL_DEFAULT_ID);
            dateIntervalDtoList.add(monthDateIntervalDto);
        }
        Map<String, List<DateIntervalDto>> dateIntervalMap = MapUtil.newHashMap(true);
        for (String refId : refIds) {
            dateIntervalMap.put(refId, dateIntervalDtoList);
        }
        List<String> secIds = this.getSecIdsByParams(params);
        // 差集求指数id集合，包括基准
        List<String> indexIds = CollectionUtil.subtractToList(secIds, refIds);
        Map<String, String> benchmarkIdMap = MapUtil.newHashMap(true);
        for (String refId : refIds) {
            if (StrUtil.isBlank(params.getBenchmarkId())) {
                continue;
            }
            benchmarkIdMap.put(refId, params.getBenchmarkId());
        }
        return CalcMultipleSecMultipleTimeRangeIndicatorReq.builder()
                .mainSecIdList(refIds)
                .secBenchmarkIdMap(benchmarkIdMap)
                .indexIdList(indexIds)
                .raiseType(params.getRaiseType())
                .strategy(StrategyHandleUtils.getStrategy(params.getStrategy()))
                .visibility(Visibility.Both)
                .dataFrequency(params.getFrequency())
                .navType(params.getNavType())
                .secDateIntervalDtoListMap(dateIntervalMap)
                .indicatorList(ListUtil.toList(Indicator.IntervalReturn))
                .geoExtraindicatorList(ListUtil.toList(Indicator.IntervalReturn))
                .ifAnnualize(true)
                .riskOfFreeId(Consts.RISK_OF_FREE)
                .calcIndexRetIndicatorValue(true)
                .ifConvertPerformanceConsistencyWord(true).build();
    }

    @Override
    protected Map<String, Object> bizSecHandle(RevenueParams params) {
        boolean ifExcessReturn = params.getIfExcessReturn() != null && params.getIfExcessReturn();
        List<String> refIds = params.getRefIds();
        List<String> secIds = this.getSecIdsByParams(params);
        List<String> indexIds = CollectionUtil.subtractToList(secIds, refIds);
        CalcMultipleSecMultipleTimeRangeIndicatorReq req = this.buildCalcReq(params, null);
        Map<String, List<IndicatorCalcPropertyDto>> secRangIndicatorMap = this.baseIndicatorServiceV2.calcMultipleSecMultipleTimeRangeIndicator(req);
        // 3、结果处�
        Map<String, Object> dataset = MapUtil.newHashMap();
        if (STATISTICS.equals(params.getRevenuType())) {
            dataset = this.getRevenueDataset(refIds, indexIds, params.getRollingFrequency(), secRangIndicatorMap, ifExcessReturn);
        } else if (DISTRIBUTION.equals(params.getRevenuType())) {
            dataset = this.loadDistributionData(refIds, indexIds, secRangIndicatorMap, Double.parseDouble(params.getSpace()), ifExcessReturn);
        }
        Map<String, String> productNameMapping = this.baseInfoService.querySecName(secIds);
        Map<String, Object> extInfos = MapUtil.<String, Object>builder("endDate", params.getEndDate()).put("startDate", params.getStartDate()).build();
        if (STATISTICS.equals(params.getRevenuType())) {
            // table
            HistoryRetTableSingleResp resp = HistoryRetTableUtil.getHistoryRetTableSingleResp(refIds, indexIds, secRangIndicatorMap, Indicator.IntervalReturn, ifExcessReturn, params.getRollingFrequency());
            extInfos.put("table", HistoryRetTableUtil.buildTableData(productNameMapping, resp, ifExcessReturn, false));
        }
        return MapUtil.<String, Object>builder().put("dataset", dataset).put("productNameMapping", productNameMapping).putAll(extInfos).build();
    }

    /**
     * 获取分布图，已支持多标的
     *
     * @param refIds         基金id，支持多标的
     * @param indexIdList    指数id列表，包括基准
     * @param dtoMap         计算返回数据
     * @param space          分布图横坐标间距
     * @param ifExcessReturn 是否计算超额
     * @return /
     */
    protected Map<String, Object> loadDistributionData(List<String> refIds, List<String> indexIdList, Map<String, List<IndicatorCalcPropertyDto>> dtoMap, double space, boolean ifExcessReturn) {
        Map<String, Object> valuesMap = MapUtil.newHashMap(true);
        Map<String, Object> indexValuesMap = MapUtil.newHashMap(true);
        double maxValue = 0d;
        double minValue = 0d;
        for (String refId : refIds) {
            List<Double> indicatorValues = dtoMap.get(refId).stream()
                    .filter(p -> !HistoryRetTableUtil.YEAR_MONTH_ROLL_LIST.contains(p.getIndicatorCalcReq().getIndicatorCalcTimeRangeDto().getId()))
                    .map(IndicatorCalcPropertyDto::getSecData)
                    .map(e -> ifExcessReturn ? e.getExtraGeoIndicatorValueMap() : e.getIndicatorValueMap())
                    .map(e -> e.get(Indicator.IntervalReturn.name())).filter(Objects::nonNull).map(Double::parseDouble).collect(Collectors.toList());
            List<Map<String, Map<String, String>>> collect = dtoMap.get(refId).stream()
                    .filter(p -> !HistoryRetTableUtil.YEAR_MONTH_ROLL_LIST.contains(p.getIndicatorCalcReq().getIndicatorCalcTimeRangeDto().getId()))
                    .map(IndicatorCalcPropertyDto::getIndexData)
                    .map(IndicatorCalcIndexDataDto::getIndexIndicatorValueMap).collect(Collectors.toList());
            maxValue = Math.max(maxValue, handleMaxAndMin(indicatorValues, (o1, o2) -> o1 > o2 ? o1 : o2));
            minValue = Math.min(minValue, handleMaxAndMin(indicatorValues, (o1, o2) -> o1 > o2 ? o2 : o1));
            for (String indexId : indexIdList) {
                List<Double> indexIndicatorValues = collect.stream().map(e -> e.get(indexId)).map(e -> e.get(Indicator.IntervalReturn.name()))
                        .filter(Objects::nonNull).map(Double::parseDouble).collect(Collectors.toList());
                maxValue = Math.max(maxValue, handleMaxAndMin(indexIndicatorValues, (o1, o2) -> o1 > o2 ? o1 : o2));
                minValue = Math.min(minValue, handleMaxAndMin(indexIndicatorValues, (o1, o2) -> o1 > o2 ? o2 : o1));
            }
        }
        for (String refId : refIds) {
            List<Double> indicatorValues = dtoMap.get(refId).stream()
                    .filter(p -> !HistoryRetTableUtil.YEAR_MONTH_ROLL_LIST.contains(p.getIndicatorCalcReq().getIndicatorCalcTimeRangeDto().getId()))
                    .map(IndicatorCalcPropertyDto::getSecData)
                    .map(e -> ifExcessReturn ? e.getExtraGeoIndicatorValueMap() : e.getIndicatorValueMap())
                    .map(e -> e.get(Indicator.IntervalReturn.name())).filter(Objects::nonNull).map(Double::parseDouble).collect(Collectors.toList());
            List<Pair<Double, Double>> dates = RangeUtils.groupNumberLimitCondition(10, space, minValue, maxValue);
            valuesMap.put(refId, DataUtil.buildValue(dates, indicatorValues));
            for (String indexId : indexIdList) {
                if (indexValuesMap.containsKey(indexId)) {
                    continue;
                }
                List<Double> indexIndicatorValues = dtoMap.get(refId).stream()
                        .filter(p -> !HistoryRetTableUtil.YEAR_MONTH_ROLL_LIST.contains(p.getIndicatorCalcReq().getIndicatorCalcTimeRangeDto().getId()))
                        .map(IndicatorCalcPropertyDto::getIndexData)
                        .map(IndicatorCalcIndexDataDto::getIndexIndicatorValueMap).map(e -> e.get(indexId)).map(e -> e.get(Indicator.IntervalReturn.name()))
                        .filter(Objects::nonNull).map(Double::parseDouble).collect(Collectors.toList());
                indexValuesMap.put(indexId, DataUtil.buildValue(dates, indexIndicatorValues));
            }
        }
        return MapUtil.<String, Object>builder().putAll(valuesMap).putAll(indexValuesMap).build();
    }

    private Map<String, Object> getRevenueDataset(List<String> refIds, List<String> indexIds, Frequency frequency, Map<String, List<IndicatorCalcPropertyDto>> secRangIndicatorMap, boolean ifExcessReturn) {
        Map<String, Object> dataset = MapUtil.newHashMap();
        Map<String, List<Map<String, Object>>> valuesMap = MapUtil.newHashMap(true);
        Map<String, List<Map<String, Object>>> indexValuesMap = MapUtil.newHashMap(true);
        List<String> dateList = ListUtil.list(false);
        for (String refId : refIds) {
            List<IndicatorCalcPropertyDto> dtos = Optional.ofNullable(secRangIndicatorMap.get(refId)).filter(e -> !e.isEmpty()).orElse(ListUtil.empty());
            // 当前基金的数据处�
            List<Map<String, Object>> fundIndicatorList = ListUtil.list(true);
            for (IndicatorCalcPropertyDto dto : dtos) {
                String id = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto).map(IndicatorCalcTimeRangeDto::getId).orElse(null);
                String date = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto).map(IndicatorCalcTimeRangeDto::getEndDate).orElse(null);
                if (id == null || HistoryRetTableUtil.YEAR_MONTH_ROLL_LIST.contains(id)) {
                    continue;
                }
                date = handleStrDate(date, frequency);
                if (!dateList.contains(date)) {
                    dateList.add(date);
                }
                String value = Optional.ofNullable(dto.getSecData()).map(IndicatorCalcSecDataDto::getIndicatorValueMap).map(e -> e.get(Indicator.IntervalReturn.name())).orElse(null);
                String extraValue = Optional.ofNullable(dto.getSecData()).map(IndicatorCalcSecDataDto::getExtraGeoIndicatorValueMap).map(e -> e.get(Indicator.IntervalReturn.name())).orElse(null);
                Map<String, Object> build = MapUtil.<String, Object>builder("date", date).put("d", date).put("extraValue", extraValue).put("value", value).build();
                fundIndicatorList.add(build);
            }
            valuesMap.put(refId, fundIndicatorList);
            // 指数的数据处�
            for (String indexId : indexIds) {
                if (indexValuesMap.containsKey(indexId)) {
                    continue;
                }
                List<Map<String, Object>> indexIndicatorList = ListUtil.list(true);
                for (IndicatorCalcPropertyDto dto : dtos) {
                    String id = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto).map(IndicatorCalcTimeRangeDto::getId).orElse(null);
                    if (id == null || HistoryRetTableUtil.YEAR_MONTH_ROLL_LIST.contains(id)) {
                        continue;
                    }
                    String date = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto).map(IndicatorCalcTimeRangeDto::getEndDate).orElse(null);
                    date = handleStrDate(date, frequency);
                    String value = Optional.ofNullable(dto.getIndexData()).map(IndicatorCalcIndexDataDto::getIndexIndicatorValueMap).map(e -> e.get(indexId)).map(e -> e.get(Indicator.IntervalReturn.name())).orElse(null);
                    indexIndicatorList.add(MapUtil.<String, Object>builder("date", date).put("value", value).build());
                }
                indexValuesMap.put(indexId, indexIndicatorList);
            }
        }
        dateList.sort(Comparator.naturalOrder());
        for (String refId : refIds) {
            List<Map<String, Object>> data = ListUtil.list(true);
            for (String date : dateList) {
                String value = valuesMap.get(refId).stream().filter(e -> date.equals(MapUtil.getStr(e, "date")))
                        .findFirst().map(e -> MapUtil.getStr(e, "value")).orElse(null);
                Map<String, Object> build = MapUtil.<String, Object>builder().put("date", date).put("value", value).build();
                if (ifExcessReturn) {
                    String extraValue = valuesMap.get(refId).stream().filter(e -> date.equals(MapUtil.getStr(e, "date")))
                            .findFirst().map(e -> MapUtil.getStr(e, "extraValue")).orElse(null);
                    build.put("extraValue", extraValue);
                }
                data.add(build);
            }
            dataset.put(refId, data);
        }
        for (String refId : indexIds) {
            List<Map<String, Object>> data = ListUtil.list(true);
            for (String date : dateList) {
                String value = indexValuesMap.get(refId).stream().filter(e -> date.equals(MapUtil.getStr(e, "date")))
                        .findFirst().map(e -> MapUtil.getStr(e, "value")).orElse(null);
                data.add(MapUtil.<String, Object>builder().put("date", date).put("value", value).build());
            }
            dataset.put(refId, data);
        }
        return dataset;
    }
}
