package com.smppw.analysis.domain.manager.performance.handler;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.performance.HistoryRetTableSingleResp;
import com.smppw.analysis.domain.dto.performance.RollingParams;
import com.smppw.analysis.domain.manager.performance.AbstractSingleSecPerformance;
import com.smppw.analysis.domain.manager.performance.HistoryRetTableUtil;
import com.smppw.analysis.domain.manager.performance.PerformanceConstants;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.analysis.infrastructure.exception.APIException;
import com.smppw.common.pojo.dto.calc.IndicatorCalcIndexDataDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcPropertyDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcReqPropertyDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcTimeRangeDto;
import com.smppw.common.pojo.dto.indicator.CalcMultipleSecMultipleTimeRangeIndicatorReq;
import com.smppw.common.pojo.dto.indicator.DateIntervalDto;
import com.smppw.common.pojo.enums.*;
import com.smppw.constants.Consts;
import com.smppw.utils.StrategyHandleUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@Component(PerformanceConstants.ROLLING)
public class RollingHandler extends AbstractSingleSecPerformance<RollingParams, Map<String, Object>> {
    public RollingHandler(BaseInfoService baseInfoService, BaseIndicatorServiceV2 baseIndicatorServiceV2) {
        super(baseInfoService, baseIndicatorServiceV2);
    }

    @Override
    protected void checkParams(RollingParams params) {
        super.checkParams(params);
        if (params.getIndicator() == null) {
            throw new APIException("分析的指标不能为空");
        }
        List<Frequency> frequencies = ListUtil.toList(Frequency.Monthly, Frequency.Quarterly, Frequency.Annually);
        if (params.getRollingFrequency() == null || !frequencies.contains(params.getRollingFrequency())) {
            logger.warn(String.format("当前滚动频率为：%s，设置为默认滚动频率：%s", params.getRollingFrequency(), Frequency.Monthly));
            params.setRollingFrequency(Frequency.Monthly);
        }
    }

    @Override
    protected CalcMultipleSecMultipleTimeRangeIndicatorReq buildCalcReq(RollingParams params, DateIntervalType dateIntervalType) {
        Indicator indicator = params.getIndicator();
        boolean excess = Indicator.INDICATOR_EXCESS_RETURN_TYPE_ARRAY.contains(indicator);
        List<Indicator> indicators = ListUtil.toList(Indicator.MaxDrawdown);
        List<Indicator> geoIndicators = ListUtil.list(false);
        if (excess) {
            geoIndicators.add(indicator);
        } else {
            indicators.add(indicator);
        }
        Frequency rollingFrequency = params.getRollingFrequency();
        List<String> refIds = params.getRefIds();
        String startDate = params.getStartDate();
        String endDate = params.getEndDate();
        List<DateIntervalDto> dateIntervalDtoList = new ArrayList<>();
        DateIntervalDto dateIntervalDto = new DateIntervalDto();
        dateIntervalDto.setDateIntervalType(dateIntervalType);
        dateIntervalDto.setStartDate(startDate);
        dateIntervalDto.setEndDate(endDate);
        dateIntervalDto.setFrequency(rollingFrequency);
        dateIntervalDtoList.add(dateIntervalDto);
        if (Frequency.Monthly == rollingFrequency || Frequency.Quarterly == rollingFrequency) {
            DateIntervalDto yearDateIntervalDto = new DateIntervalDto();
            yearDateIntervalDto.setDateIntervalType(dateIntervalType);
            yearDateIntervalDto.setStartDate(startDate);
            yearDateIntervalDto.setEndDate(endDate);
            yearDateIntervalDto.setFrequency(Frequency.Annually);
            yearDateIntervalDto.setId(HistoryRetTableUtil.YEAR_ROLL_DEFAULT_ID);
            dateIntervalDtoList.add(yearDateIntervalDto);
        } else if (Frequency.Weekly == rollingFrequency) {
            DateIntervalDto monthDateIntervalDto = new DateIntervalDto();
            monthDateIntervalDto.setDateIntervalType(dateIntervalType);
            monthDateIntervalDto.setStartDate(startDate);
            monthDateIntervalDto.setEndDate(endDate);
            monthDateIntervalDto.setFrequency(Frequency.Monthly);
            monthDateIntervalDto.setId(HistoryRetTableUtil.MONTH_ROLL_DEFAULT_ID);
            dateIntervalDtoList.add(monthDateIntervalDto);
        }
        Map<String, List<DateIntervalDto>> dateIntervalMap = MapUtil.newHashMap(true);
        for (String refId : refIds) {
            dateIntervalMap.put(refId, dateIntervalDtoList);
        }
        List<String> secIds = this.getSecIdsByParams(params);
        // 差集求指数id集合，包括基准
        List<String> indexIds = CollectionUtil.subtractToList(secIds, refIds);
        Map<String, String> benchmarkIdMap = MapUtil.newHashMap(true);
        for (String refId : refIds) {
            if (StrUtil.isBlank(params.getBenchmarkId())) {
                continue;
            }
            benchmarkIdMap.put(refId, params.getBenchmarkId());
        }
        return CalcMultipleSecMultipleTimeRangeIndicatorReq.builder()
                .mainSecIdList(refIds)
                .secBenchmarkIdMap(benchmarkIdMap)
                .indexIdList(indexIds)
                .raiseType(params.getRaiseType())
                .strategy(StrategyHandleUtils.getStrategy(params.getStrategy()))
                .visibility(Visibility.Both)
                .dataFrequency(params.getFrequency())
                .navType(params.getNavType())
                .secDateIntervalDtoListMap(dateIntervalMap)
                .indicatorList(indicators)
                .geoExtraindicatorList(geoIndicators)
                .ifAnnualize(true)
                .riskOfFreeId(Consts.RISK_OF_FREE)
                .calcIndexRetIndicatorValue(true)
                .ifConvertPerformanceConsistencyWord(true).build();
    }

    @Override
    protected Map<String, Object> bizSecHandle(RollingParams params) {
        List<String> refIds = params.getRefIds();
        List<String> secIds = this.getSecIdsByParams(params);
        List<String> indexIds = CollectionUtil.subtractToList(secIds, refIds);
        Indicator indicator = params.getIndicator();
        Frequency rollingFrequency = params.getRollingFrequency();
        boolean excess = Indicator.INDICATOR_EXCESS_RETURN_TYPE_ARRAY.contains(indicator);
        CalcMultipleSecMultipleTimeRangeIndicatorReq req = this.buildCalcReq(params, DateIntervalType.DefaultRolling);
        Map<String, List<IndicatorCalcPropertyDto>> indicatorMap = this.baseIndicatorServiceV2.calcMultipleSecMultipleTimeRangeIndicator(req);
        Map<TimeRange, Boolean> timeRangeFlag = MapUtil.newHashMap();
        Map<String, Object> valuesMap = MapUtil.newHashMap(true);
        Map<String, Object> indexValuesMap = MapUtil.newHashMap(true);
        for (String refId : refIds) {
            List<IndicatorCalcPropertyDto> dtos = Optional.ofNullable(indicatorMap.get(refId)).filter(e -> !e.isEmpty()).orElse(ListUtil.empty());
            // 当前基金的数据处理
            List<Map<String, Object>> fundIndicatorList = getFundIndicatorList(params, indicator, excess, rollingFrequency, timeRangeFlag, dtos);
            valuesMap.put(refId, fundIndicatorList);
            // 指数的数据处理
            if (excess) {
                continue;
            }
            handleIndexData(indicator, rollingFrequency, indexIds, timeRangeFlag, indexValuesMap, dtos);
        }
        Map<String, Object> dataset = MapUtil.<String, Object>builder().putAll(valuesMap).putAll(indexValuesMap).build();
        Map<String, String> productNameMapping = this.baseInfoService.querySecName(secIds);
        if (excess) {
            for (String indexId : indexIds) {
                productNameMapping.remove(indexId);
            }
        }
        Map<String, Object> extInfos = MapUtil.<String, Object>builder("endDate", params.getEndDate()).put("startDate", params.getStartDate()).build();
        HistoryRetTableSingleResp resp = HistoryRetTableUtil.getHistoryRetTableSingleResp(refIds, indexIds, indicatorMap, indicator, excess, rollingFrequency);
        extInfos.put("table", HistoryRetTableUtil.buildTableData(productNameMapping, resp, excess, true));
        return MapUtil.<String, Object>builder().put("dataset", dataset).put("productNameMapping", productNameMapping).putAll(extInfos).build();
    }

    private void handleIndexData(Indicator indicator, Frequency rollingFrequency, List<String> indexIds,
                                 Map<TimeRange, Boolean> timeRangeFlag, Map<String, Object> indexValuesMap, List<IndicatorCalcPropertyDto> dtos) {
        for (String indexId : indexIds) {
            if (indexValuesMap.containsKey(indexId)) {
                continue;
            }
            List<Map<String, Object>> indexIndicatorList = ListUtil.list(true);
            for (IndicatorCalcPropertyDto dto : dtos) {
                String id = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto).map(IndicatorCalcTimeRangeDto::getId).orElse(null);
                String date = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto).map(IndicatorCalcTimeRangeDto::getEndDate).orElse(null);
                TimeRange timeRange = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto).map(IndicatorCalcTimeRangeDto::getTimeRange).orElse(null);
                boolean yearMonthFlag = (id != null && HistoryRetTableUtil.YEAR_MONTH_ROLL_LIST.contains(id));
                boolean tf = timeRange == null || timeRangeFlag.getOrDefault(timeRange, false);
                boolean flag = yearMonthFlag || tf;
                if (flag || date == null) {
                    continue;
                }
                String value = Optional.ofNullable(dto.getIndexData()).map(IndicatorCalcIndexDataDto::getIndexIndicatorValueMap).map(e -> e.get(indexId)).map(e -> e.get(indicator.name())).orElse(null);
                indexIndicatorList.add(MapUtil.<String, Object>builder("date", handleStrDate(date, rollingFrequency)).put("value", value).build());
            }
            indexValuesMap.put(indexId, indexIndicatorList);
        }
    }

    private List<Map<String, Object>> getFundIndicatorList(RollingParams params, Indicator indicator, boolean excess, Frequency rollingFrequency,
                                                           Map<TimeRange, Boolean> timeRangeFlag, List<IndicatorCalcPropertyDto> dtos) {
        List<Map<String, Object>> fundIndicatorList = ListUtil.list(true);
        for (IndicatorCalcPropertyDto dto : dtos) {
            String date = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto).map(IndicatorCalcTimeRangeDto::getEndDate).orElse(null);
            String id = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto).map(IndicatorCalcTimeRangeDto::getId).orElse(null);
            TimeRange timeRange = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto).map(IndicatorCalcTimeRangeDto::getTimeRange).orElse(null);
            boolean flag = (id != null && HistoryRetTableUtil.YEAR_MONTH_ROLL_LIST.contains(id));
            if (timeRange == null || flag) {
                continue;
            }
            if (params.getTimeRange() != null && params.getTimeRange() == TimeRange.Custom) {
                String start = Optional.ofNullable(dto.getIndicatorCalcReq()).map(IndicatorCalcReqPropertyDto::getIndicatorCalcTimeRangeDto).map(IndicatorCalcTimeRangeDto::getStartDate).orElse(null);
                boolean isNotNull = params.getStartDate() != null && start != null;
                if (isNotNull && params.getStartDate().compareTo(start) > 0) {
                    timeRangeFlag.put(timeRange, true);
                    continue;
                }
            }
            boolean isFromSetup = params.getTimeRange() != null && params.getTimeRange() != TimeRange.FromSetup;
            boolean isFromThisYear = params.getTimeRange() != null && params.getTimeRange() != TimeRange.FromThisYear;
            boolean isCustom = params.getTimeRange() != null && params.getTimeRange() != TimeRange.Custom;
            boolean isOver = params.getTimeRange() != null && params.getTimeRange().getId() < timeRange.getId();
            boolean isTimaRange = isFromSetup && isFromThisYear && isCustom && isOver;
            if (isTimaRange || date == null) {
                timeRangeFlag.put(timeRange, true);
                continue;
            }
            String value = Optional.ofNullable(dto.getSecData()).map(e -> excess ? e.getExtraGeoIndicatorValueMap() : e.getIndicatorValueMap()).map(e -> e.get(indicator.name())).orElse(null);
            fundIndicatorList.add(MapUtil.<String, Object>builder("date", handleStrDate(date, rollingFrequency)).put("value", value).build());
        }
        return fundIndicatorList;
    }
}
