package com.smppw.analysis.domain.manager.performance.handler;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.performance.TrendParams;
import com.smppw.analysis.domain.manager.performance.AbstractSingleSecPerformance;
import com.smppw.analysis.domain.manager.performance.PerformanceConstants;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.common.pojo.dto.calc.IndicatorCalcIndexDataDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcPropertyDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcSecDataDto;
import com.smppw.common.pojo.dto.indicator.DateIntervalDto;
import com.smppw.common.pojo.enums.DateIntervalType;
import com.smppw.common.pojo.enums.TrendType;
import com.smppw.common.pojo.enums.Visibility;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Optional;

@Component(PerformanceConstants.TREND)
public class TrendHandler extends AbstractSingleSecPerformance<TrendParams, Map<String, Object>> {
    public TrendHandler(BaseInfoService baseInfoService, BaseIndicatorServiceV2 baseIndicatorServiceV2) {
        super(baseInfoService, baseIndicatorServiceV2);
    }

    @Override
    protected Map<String, Object> bizSecHandle(TrendParams params) {
        List<String> refIds = params.getRefIds();
        List<String> secIds = this.getSecIdsByParams(params);
        List<String> indexIds = CollectionUtil.subtractToList(secIds, refIds);
        // 时段参数构建
        DateIntervalDto dateInterval = this.buildDateIntervalByParams(params, DateIntervalType.CustomInterval, params.getFrequency());
        Map<String, String> benchmarkIdMap = MapUtil.newHashMap(true);
        for (String refId : refIds) {
            if (StrUtil.isBlank(params.getBenchmarkId())) {
                continue;
            }
            benchmarkIdMap.put(refId, params.getBenchmarkId());
        }
        List<TrendType> trendTypes = params.getTrendTypes();
        List<TrendType> indexTrendTypes = params.getIndexTrendTypes();
        // 计算走势图
        Map<String, List<IndicatorCalcPropertyDto>> trendMap = this.baseIndicatorServiceV2.getMultipleSecTrend(refIds, benchmarkIdMap,
                indexIds, dateInterval, params.getFrequency(), null, null, Visibility.Both, params.getNavType(), trendTypes);
        // 结果处理，支持多标的
        // 处理走势图
        Map<String, List<Map<String, Object>>> dataListMap = MapUtil.newHashMap(true);
        Map<String, List<Map<String, Object>>> indexDataListMap = MapUtil.newHashMap(true);
        Map<String, Map<TrendType, List<Double>>> trendListMap = MapUtil.newHashMap(true);
        for (String refId : refIds) {
            IndicatorCalcPropertyDto dto = Optional.ofNullable(trendMap.get(refId)).filter(e -> !e.isEmpty()).map(e -> e.get(0)).orElse(null);
            List<String> tempDateList = Optional.ofNullable(dto).map(IndicatorCalcPropertyDto::getDateList).orElse(ListUtil.empty());
            // 基金走势序列
            Map<TrendType, List<Double>> trendTypeListMap = Optional.ofNullable(dto).map(IndicatorCalcPropertyDto::getSecData).map(IndicatorCalcSecDataDto::getTrendValueMap).orElse(MapUtil.empty());
            trendListMap.put(refId, trendTypeListMap);
            dataListMap.put(refId, this.buildDateValue(tempDateList, trendTypeListMap, trendTypes));
            // 指数净值和收益序列
            Map<String, Map<TrendType, List<Double>>> tempIndexTrendTypeListMap = Optional.ofNullable(dto).map(IndicatorCalcPropertyDto::getIndexData).map(IndicatorCalcIndexDataDto::getIndexTrendValueMap).orElse(MapUtil.empty());
            for (String indexId : indexIds) {
                if (indexDataListMap.containsKey(indexId) && !indexId.equals(params.getBenchmarkId())) {
                    continue;
                }
                indexDataListMap.put(indexId, this.buildDateValue(tempDateList, tempIndexTrendTypeListMap.get(indexId), indexTrendTypes));
            }
        }
        Map<String, String> productNameMapping = this.baseInfoService.querySecName(secIds);
        Map<String, List<Map<String, Object>>> dataset = MapUtil.<String, List<Map<String, Object>>>builder().putAll(dataListMap).build();
        Map<String, Object> extInfos = MapUtil.<String, Object>builder().put("endDate", params.getEndDate()).put("startDate", params.getStartDate()).build();
        for (String refId : refIds) {
            Map<TrendType, List<Double>> trendTypeListMap = trendListMap.getOrDefault(refId, MapUtil.empty());
            Double maxDown = handleMaxAndMin(trendTypeListMap.get(TrendType.DrawdownTrend), (o1, o2) -> o1 > o2 ? o2 : o1);
            Double maxExtraDown = handleMaxAndMin(trendTypeListMap.get(TrendType.ExtraDrawdownTrend), (o1, o2) -> o1 > o2 ? o2 : o1);
            Map<String, Object> data = MapUtil.<String, Object>builder().put("maxDown", maxDown).put("maxExtraDown", maxExtraDown).build();
            extInfos.put(refId, data);
        }
        return MapUtil.<String, Object>builder().put("dataset", dataset).put("productNameMapping", productNameMapping).putAll(extInfos).build();
    }

    /**
     * 构建日期值对象
     *
     * @param dateList         日期序列
     * @param trendTypeListMap 走势图数据
     * @param trendTypes       走势图类型列表
     * @return /
     */
    private List<Map<String, Object>> buildDateValue(List<String> dateList, Map<TrendType, List<Double>> trendTypeListMap, List<TrendType> trendTypes) {
        List<Map<String, Object>> dataList = ListUtil.list(true);
        for (int i = 0; i < dateList.size(); i++) {
            Map<String, Object> temp = MapUtil.<String, Object>builder("date", dateList.get(i)).build();
            for (TrendType trendType : trendTypes) {
                List<Double> doubles = trendTypeListMap.get(trendType);
                Object value = null;
                try {
                    value = i <= doubles.size() ? doubles.get(i) : null;
                } catch (Exception ignored) {
                }
                temp.put(StrUtil.toCamelCase(trendType.name()), value);
            }
            dataList.add(temp);
        }
        return dataList;
    }
}
