package com.smppw.analysis.domain.manager.performance.handler;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.performance.WinParams;
import com.smppw.analysis.domain.dto.performance.WinVO;
import com.smppw.analysis.domain.manager.performance.AbstractSingleSecPerformance;
import com.smppw.analysis.domain.manager.performance.PerformanceConstants;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.analysis.infrastructure.consts.WinRateBmk;
import com.smppw.analysis.infrastructure.exception.APIException;
import com.smppw.common.pojo.dto.CompoundRet;
import com.smppw.common.pojo.dto.calc.IndicatorCalcPropertyDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcSecDataDto;
import com.smppw.common.pojo.dto.indicator.CalcMultipleSecMultipleTimeRangeIndicatorReq;
import com.smppw.common.pojo.enums.DateIntervalType;
import com.smppw.common.pojo.enums.Indicator;
import com.smppw.constants.Consts;
import com.smppw.utils.BigDecimalUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

@Component(PerformanceConstants.WIN)
public class WinHandler extends AbstractSingleSecPerformance<WinParams, WinVO> {
    public WinHandler(BaseInfoService baseInfoService, BaseIndicatorServiceV2 baseIndicatorServiceV2) {
        super(baseInfoService, baseIndicatorServiceV2);
    }

    @Override
    protected void checkParams(WinParams params) {
        super.checkParams(params);
        if (params.getRefIds().size() > 1) {
            logger.warn("多标的不支持胜率分析！");
        }
    }

    @Override
    protected CalcMultipleSecMultipleTimeRangeIndicatorReq buildCalcReq(WinParams params, DateIntervalType dateIntervalType) {
        CalcMultipleSecMultipleTimeRangeIndicatorReq req = super.buildCalcReq(params, dateIntervalType);
        req.setIndicatorList(ListUtil.toList(Indicator.IntervalReturn, Indicator.WinRate));
        boolean winZero = params.getWinRateBmk() == WinRateBmk.Zero;
        req.setRiskOfFreeId(Consts.RISK_OF_FREE);
        req.setIndexIdList(ListUtil.empty());
        req.setNeedRet(true);
        req.setNeedExtraRet(true);
        if (!winZero) {
            req.setGeoExtraindicatorList(ListUtil.toList(Indicator.WinRate));
        }
        return req;
    }

    @Override
    protected WinVO bizSecHandle(WinParams params) {
        String fundId = params.getRefIds().get(0);
        Indicator indicator = Indicator.WinRate;
        boolean winZero = params.getWinRateBmk() == WinRateBmk.Zero;
        CalcMultipleSecMultipleTimeRangeIndicatorReq req = this.buildCalcReq(params, DateIntervalType.CustomInterval);
        Map<String, List<IndicatorCalcPropertyDto>> indicatorMap = this.baseIndicatorServiceV2.calcMultipleSecMultipleTimeRangeIndicator(req);
        IndicatorCalcSecDataDto secData = Optional.ofNullable(indicatorMap.get(fundId)).filter(e -> !e.isEmpty()).map(e -> e.get(0))
                .map(IndicatorCalcPropertyDto::getSecData).orElseThrow(() -> new APIException("计算错误"));
        String winRate = winZero ? secData.getIndicatorValueMap().get(indicator.name()) : secData.getExtraGeoIndicatorValueMap().get(indicator.name());
        List<CompoundRet> retList = secData.getRetList();
        // 返回结果对象构建
        WinVO.WinData winData = this.prepareWinRateData("allTime", winRate, retList, winZero);
        Double lower = Optional.ofNullable(params.getLower()).map(Double::parseDouble).map(e -> e / 100).orElse(-0.03d);
        Double upper = Optional.ofNullable(params.getUpper()).map(Double::parseDouble).map(e -> e / 100).orElse(0.03d);
        List<CompoundRet> riseRetList = retList.stream().filter(e -> e.getRetOfBmk() != null && e.getRetOfBmk() >= upper).collect(Collectors.toList());
        List<CompoundRet> fallRetList = retList.stream().filter(e -> e.getRetOfBmk() != null && e.getRetOfBmk() <= lower).collect(Collectors.toList());
        List<CompoundRet> shockRetList = retList.stream().filter(e -> e.getRetOfBmk() != null && e.getRetOfBmk() <= upper && e.getRetOfBmk() >= lower).collect(Collectors.toList());
        List<WinVO.WinData> marketWinData = ListUtil.list(true);
        Map<String, List<CompoundRet>> timeMap = MapUtil.<String, List<CompoundRet>>builder(MapUtil.newHashMap(true))
                .put("allTime", retList).put("riseTime", riseRetList).put("shockTime", shockRetList).put("fallTime", fallRetList).build();
        if (winZero) {
            timeMap.remove("shockTime");
        }
        timeMap.forEach((k, v) -> marketWinData.add(this.prepareWinRateData(k, winRate, v, winZero)));
        return new WinVO(winData, marketWinData);
    }

    /**
     * 胜率业务处理，从老项目迁移过来
     *
     * @param id      /
     * @param winRate /
     * @param retList /
     * @param winZero /
     * @return /
     */
    protected WinVO.WinData prepareWinRateData(String id, String winRate, List<CompoundRet> retList, boolean winZero) {
        Map<String, String> timeMap = MapUtil.builder("allTime", "全时段").put("riseTime", "上涨周期")
                .put("fallTime", "下跌周期").put("shockTime", "震荡周期").build();
        List<Double> fundList = ListUtil.list(false);
        List<Double> bmkList = ListUtil.list(false);
        int winItem = 0;
        int lossItem = 0;
        int totalItem = 0;
        for (CompoundRet compoundRet : retList) {
            if (compoundRet.getRetOfBmk() == null) {
                if (winZero && compoundRet.getRetOfFund() != null) {
                    compoundRet.setRetOfBmk(0.d);
                } else {
                    continue;
                }
            }
            fundList.add(compoundRet.getRetOfFund());
            bmkList.add(compoundRet.getRetOfBmk());
            if (winZero) {
                if (compoundRet.getRetOfFund() != null) {
                    if ((compoundRet.getRetOfFund() - 0) > 0) {
                        winItem++;
                    } else {
                        lossItem++;
                    }
                    totalItem++;
                }
            } else {
                if (compoundRet.getRetOfFund() != null && compoundRet.getRetOfBmk() != null) {
                    if (compoundRet.getRetOfFund() - compoundRet.getRetOfBmk() > 0) {
                        winItem++;
                    } else {
                        lossItem++;
                    }
                    totalItem++;
                }
            }
        }

        String rate = winRate;
        if (!"allTime".equals(id)) {
            rate = totalItem == 0 ? "0" : BigDecimalUtils.divide(new BigDecimal(winItem), new BigDecimal(totalItem))
                    .setScale(4, RoundingMode.HALF_UP).toString();
        }
        Double meanFundRet = meanRetCal(fundList);
        Double meanBenchmarkRet = meanRetCal(bmkList);
        Double excessMeanRet = null;
        if (null != meanFundRet && null != meanBenchmarkRet) {
            excessMeanRet = meanFundRet - meanBenchmarkRet;
        }
        return WinVO.WinData.builder().dateType(timeMap.get(id))
                .totalItem(String.valueOf(totalItem))
                .winItem(String.valueOf(winItem))
                .lossItem(String.valueOf(lossItem))
                .meanRet(StrUtil.toStringOrNull(meanFundRet))
                .benchmarkMeanRet(StrUtil.toStringOrNull(meanBenchmarkRet))
                .excessMeanRet(StrUtil.toStringOrNull(excessMeanRet))
                .winRate(rate)
                .build();
    }

    private Double meanRetCal(List<Double> list) {
        double sum = 0.0;
        int couter = 0;

        for (Double d : list) {
            if (null != d) {
                sum += d;
                couter++;
            }
        }

        if (couter < 1) {
            return null;
        }

        return sum / couter;
    }
}
