package com.smppw.analysis.domain.manager.position;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.smppw.analysis.domain.dto.position.*;
import com.smppw.analysis.domain.dto.position.stock.StockAllocationVO;
import com.smppw.analysis.domain.dataobject.IndexSecWeightInfoDO;
import com.smppw.analysis.domain.dataobject.SwSecIndustryInfoDO;
import com.smppw.analysis.domain.event.SaveCacheEvent;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.domain.dao.PositionAnalysisDao;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.analysis.infrastructure.consts.RedisConst;
import com.smppw.analysis.infrastructure.exception.APIException;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.constants.SecType;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.Executor;
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/7/14 18:14
 * @description 有基准的业绩分析
 */
public abstract class AbstractAnalysisBizHandler<P extends BaseAnalysisParams, R> extends AbstractNonSynthesizeBizHandler<P, R> {
    protected static final Executor DEFAULT_EXECUTOR = ForkJoinPool.commonPool();
    protected final PositionAnalysisDao analysisService;

    public AbstractAnalysisBizHandler(AnalysisProperty property, CacheFactory cacheFactory,
                                      PositionLoadFactory factory, PositionAnalysisDao analysisService) {
        super(property, cacheFactory, factory);
        this.analysisService = analysisService;
    }

    @Override
    protected void checkParams(P params) {
        super.checkParams(params);
        if (CharSequenceUtil.isBlankOrUndefined(params.getBenchmarkId())) {
            throw new APIException("基准不能为空！");
        }
    }

    /**
     * 用来过滤大类的钩子函数，可以区分股票、期货和债券，以及后续可能新增的基金
     *
     * @return /
     */
    protected AssetCategoryEnum filterAsset() {
        return AssetCategoryEnum.STOCK;
    }

    @Override
    protected R afterNonSynthesizeLoadData(P params, List<FundPositionBaseInfo> baseInfos, List<FundPositionDetail> positionInfos) {
        this.current = System.currentTimeMillis();
        // 得到持仓信息
        List<FundPositionDetail> dataList = positionInfos.stream().filter(e -> SecType.PUBLICLY_OFFERED_FUNDS.equals(e.getFundType()) || (e.getLevel() != null && e.getLevel() >= 4))
                .filter(e -> this.filterAsset().name().equals(e.getAsset().getValue())).collect(Collectors.toList());
        if (CollUtil.isEmpty(positionInfos)) {
            return null;
        }
        String benchmarkId = params.getBenchmarkId();
        // 日期升序
        List<String> dateList = positionInfos.stream().map(FundPositionDetail::getDate).distinct().sorted().collect(Collectors.toList());
        // 基准每个估值日的权重数据
        Map<String, List<IndexSecWeightInfoDO>> indexSecWeightMap = this.getSecDateWeight(benchmarkId, dateList);
        // 数据处理
        return this.afterAnalysisLoadData(benchmarkId, dataList, indexSecWeightMap);
    }

    /**
     * 获取行业与股票对应关系 (A股)
     *
     * @return /
     */
    protected Map<ValueLabelVO, List<String>> getIndustrySecMap() {
        String key = RedisConst.POSITION_STOCK_INDUSTRY_KEY;
        Object obj = this.cacheGateway.get(key);
        List<SwSecIndustryInfoDO> dataList = ListUtil.list(false);
        if (obj != null) {
            dataList = this.convertList(obj, SwSecIndustryInfoDO.class);
        } else {
            List<SwSecIndustryInfoDO> industryInfos = this.analysisService.listSecIndustryInfo();
            if (industryInfos != null) {
                dataList.addAll(industryInfos);
            }
            SaveCacheEvent<List<SwSecIndustryInfoDO>> event = new SaveCacheEvent<>(key, industryInfos,
                    t -> this.cacheGateway.set(key, t, RedisConst.POSITION_DAY_TTL, TimeUnit.SECONDS));
            this.applicationContext.publishEvent(event);
        }
        return dataList.stream().collect(Collectors.groupingBy(e -> new ValueLabelVO(e.getIndustryCode(), e.getIndustryName()),
                Collectors.mapping(SwSecIndustryInfoDO::getSecCode, Collectors.toList())));
    }

    /**
     * 获取指数权重
     *
     * @param benchmarkId 基准id
     * @param dateList    估值日期序列
     * @return 对应日期的指数成分股占比
     */
    private Map<String, List<IndexSecWeightInfoDO>> getSecDateWeight(String benchmarkId, List<String> dateList) {
        String key = RedisConst.POSITION_INDEX_WEIGHT_KEY + benchmarkId;
        Map<String, List<IndexSecWeightInfoDO>> indexSecDateWeightMap = MapUtil.newHashMap(dateList.size());
        Map<String, Object> hmget = this.cacheGateway.hget(key);
        hmget.forEach((k, v) -> {
            if (dateList.contains(k)) {
                List<IndexSecWeightInfoDO> tempList = this.convertList(v, IndexSecWeightInfoDO.class);
                indexSecDateWeightMap.put(k, tempList);
            }
        });

        // 过滤不存在的日期的数据，重新从数据库获取并放入缓存
        List<String> dates = hmget.keySet().stream().map(Object::toString).distinct().collect(Collectors.toList());
        List<String> realDates = CollUtil.subtractToList(dateList, dates);
        for (String date : realDates) {
            List<IndexSecWeightInfoDO> indexSecWeightList;
            Object hget = this.cacheGateway.hget(key, date);
            if (hget == null) {
                indexSecWeightList = this.analysisService.listIndexSecWeight(benchmarkId, date);
                SaveCacheEvent<List<IndexSecWeightInfoDO>> event = new SaveCacheEvent<>(key, indexSecWeightList,
                        t -> this.cacheGateway.hset(key, date, t, RedisConst.POSITION_MONTH_TTL, TimeUnit.SECONDS));
                this.applicationContext.publishEvent(event);
            } else {
                indexSecWeightList = this.convertList(hget, IndexSecWeightInfoDO.class);
            }
            indexSecDateWeightMap.putIfAbsent(date, indexSecWeightList);
        }
        return indexSecDateWeightMap;
    }

    /**
     * 得到数据化的逻辑处理
     *
     * @param benchmarkId     基准id
     * @param positionDetails 持仓详情
     * @param indexWeightMap  指数权重
     * @return /
     */
    protected abstract R afterAnalysisLoadData(String benchmarkId, List<FundPositionDetail> positionDetails,
                                               Map<String, List<IndexSecWeightInfoDO>> indexWeightMap);

    /**
     * 构建分析结果集
     *
     * @param positionDetails 当前日期下的
     * @param total           股票总市值
     * @param categoryCodes   有分类的所有股票代码
     * @param secWeightList   指数成分占比
     * @param secCodes        分类下的所有股票代码
     * @return /
     */
    protected CategoryConstraint buildCategoryConstraint(List<FundPositionDetail> positionDetails, BigDecimal total, List<String> categoryCodes,
                                                         List<IndexSecWeightInfoDO> secWeightList, List<String> secCodes) {
        if (categoryCodes == null) {
            categoryCodes = ListUtil.list(false);
        }
        CategoryConstraint constraint = new CategoryConstraint();
        BigDecimal pupil = BigDecimal.ZERO;
        if (total != null && total.compareTo(BigDecimal.ZERO) > 0) {
            // 找到同风格的基金成分
            BigDecimal indexWeight = secWeightList.stream().filter(e -> secCodes.contains(e.getSecCode())).map(IndexSecWeightInfoDO::getWeight)
                    .reduce(BigDecimal::add).map(e -> e.divide(BigDecimal.valueOf(100), 6, RoundingMode.HALF_UP)).orElse(BigDecimal.ZERO);
            List<FundPositionDetail> collect = positionDetails.stream().filter(e -> secCodes.contains(e.getRef().getValue())).collect(Collectors.toList());
            categoryCodes.addAll(collect.stream().map(e -> e.getRef().getValue()).distinct().collect(Collectors.toList()));
            BigDecimal bull = collect.stream().filter(e -> Objects.equals(1, e.getSharesNature())).map(FundPositionDetail::getMarketValue)
                    .reduce(BigDecimal::add).map(e -> e.divide(total, 6, RoundingMode.HALF_UP)).orElse(BigDecimal.ZERO);
            BigDecimal bear = collect.stream().filter(e -> Objects.equals(2, e.getSharesNature())).map(FundPositionDetail::getMarketValue).reduce(BigDecimal::add)
                    .map(BigDecimal::abs).map(e -> BigDecimal.valueOf(-1).multiply(e)).map(e -> e.divide(total, 6, RoundingMode.HALF_UP)).orElse(BigDecimal.ZERO);
            constraint.setBull(bull);
            constraint.setBear(bear);
            // 多空
            pupil = bull.add(bear);
            constraint.setBenchmark(indexWeight);
        }
        constraint.setPupil(pupil);
        return constraint;
    }

    /**
     * 返回归因分析结果，包括处理其他分类
     *
     * @param date               当前估值日期
     * @param positionDetails    当前估值日期的持仓数据
     * @param total              总市值
     * @param indexSecWeightList 指数成分占比
     * @param constraintList     分析结果集
     * @param categoryCodes      所有已有的分类股票代码集合，用来获取other代码
     * @return /
     */
    protected StockAllocationVO getStockAllocationVO(String date, List<FundPositionDetail> positionDetails,
                                                     BigDecimal total, List<IndexSecWeightInfoDO> indexSecWeightList,
                                                     List<CategoryConstraint> constraintList, List<String> categoryCodes) {
        List<String> secCodes = positionDetails.stream().map(e -> e.getRef().getValue()).distinct().collect(Collectors.toList());
        List<String> otherCodes = CollUtil.subtractToList(secCodes, categoryCodes);
        if (CollUtil.isNotEmpty(otherCodes)) {
            CategoryConstraint constraint = this.buildCategoryConstraint(positionDetails, total, categoryCodes, indexSecWeightList, otherCodes);
            constraint.setCategory(OTHER);
            constraintList.add(constraint);
        }
        // 按多空降序
        constraintList.sort(((o1, o2) -> o2.getPupil().compareTo(o1.getPupil())));
        StockAllocationVO vo = new StockAllocationVO();
        vo.setDate(date);
        vo.setIndustries(constraintList);
        return vo;
    }
}
