package com.smppw.analysis.domain.manager.position;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.smppw.analysis.domain.dto.position.BaseParams;
import com.smppw.analysis.domain.dto.position.FundPositionBaseInfo;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.dto.position.PositionConstants;
import com.smppw.analysis.domain.event.SaveCacheEvent;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.domain.gateway.CacheGateway;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.analysis.infrastructure.consts.RedisConst;
import com.smppw.analysis.infrastructure.exception.APIException;
import com.smppw.common.pojo.ValueLabelVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.lang.NonNull;

import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/7 15:04
 * @description 抽象的业务处理器，加载数据用本地缓存存储
 */
public abstract class AbstractBizHandler<P extends BaseParams, R> implements BizHandler<P, R>, ApplicationContextAware {
    protected static final ValueLabelVO OTHER = PositionConstants.OTHER_ASSET;
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());
    protected final CacheGateway<Object> cacheGateway;
    protected final PositionLoadFactory factory;
    protected long current;
    protected ApplicationContext applicationContext;
    private long executeTime;

    public AbstractBizHandler(AnalysisProperty property, CacheFactory cacheFactory, PositionLoadFactory factory) {
        this.factory = factory;
        this.cacheGateway = cacheFactory.getCacheGateway(property.getCacheType());
    }

    @Override
    public void setApplicationContext(@NonNull ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }

    /**
     * 模板方法，是否返回所有数据，默认否。否的时候做时间过滤
     *
     * @return /
     */
    protected boolean isAllPositionData() {
        return false;
    }

    @Override
    public long getExecuteTime() {
        return this.executeTime;
    }

    /**
     * 参数校验，子类可以覆盖重写
     *
     * @param params 请求参数
     */
    protected void checkParams(P params) {
        if (CharSequenceUtil.isBlankOrUndefined(params.getFundId())) {
            throw new APIException("基金id不能为空！");
        }
        if (CharSequenceUtil.isBlankOrUndefined(params.getStartDate()) || CharSequenceUtil.isBlankOrUndefined(params.getEndDate())) {
            throw new APIException("时间区间不能为空！");
        }
    }

    /**
     * 一个简单的数据转换，object对象转list
     *
     * @param obj   待转换对象
     * @param clazz 泛型类型
     * @param <T>   类型参数
     * @return 对应泛型类型的实体的集合
     */
    protected <T> List<T> convertList(Object obj, Class<T> clazz) {
        try {
            return JSONUtil.toList(JSONUtil.toJsonStr(obj), clazz);
        } catch (Exception e) {
            logger.warn(StrUtil.format("数据转换list失败：{}", e.getMessage()));
        }
        return ListUtil.empty();
    }

    /**
     * 一个简单的数据转换，object对象转bean
     *
     * @param obj   待转换对象
     * @param clazz 泛型类型
     * @param <T>   类型参数
     * @return 对应泛型类型实体
     */
    protected <T> T convertBean(Object obj, Class<T> clazz) {
        try {
            return JSONUtil.toBean(JSONUtil.toJsonStr(obj), clazz);
        } catch (Exception e) {
            logger.warn(StrUtil.format("数据转换实体失败：{}", e.getMessage()));
        }
        return null;
    }

    /**
     * 接口业务处理
     *
     * @param params 请求参数
     * @return R类型数据
     */
    @Override
    public R bizHandle(P params) {
        this.current = System.currentTimeMillis();
        try {
            this.checkParams(params);
            List<FundPositionBaseInfo> fundPositionBaseInfos = this.loadFundPositionBaseInfo(params);
            List<FundPositionDetail> positionInfos = this.loadPositionData(params);
            if (!this.isAllPositionData() && CollUtil.isNotEmpty(positionInfos)) {
                // 数量变化处理后时段过滤
                positionInfos = positionInfos.stream().filter(e -> params.getStartDate().compareTo(e.getDate()) <= 0)
                        .filter(e -> params.getEndDate().compareTo(e.getDate()) >= 0).collect(Collectors.toList());
            }
            if (CollUtil.isNotEmpty(positionInfos)) {
                positionInfos = positionInfos.stream().filter(e -> e.getRef() != null).collect(Collectors.toList());
            }
            return this.afterLoadData(params, fundPositionBaseInfos, positionInfos);
        } catch (Exception e) {
            logger.warn(String.format("接口【%s】请求错误：%s", JSONUtil.toJsonStr(params), e.getMessage()));
            return null;
        } finally {
            // 会不会获取到别的线程的数据？
            this.executeTime = System.currentTimeMillis() - this.current;
        }
    }

    /**
     * 提供给子类实现的数据转换逻辑接口
     *
     * @param params        请求参数
     * @param baseInfos     基金的基本持仓信息，总资产和净资产
     * @param positionInfos 获取到的所有持仓信息，过滤了资产大类为null、成分为null的数据
     * @return /
     */
    protected abstract R afterLoadData(P params, List<FundPositionBaseInfo> baseInfos, List<FundPositionDetail> positionInfos);

    /**
     * 同步的获取基金的持仓信息，获取后放入缓存
     * synchronized 关键字被优化过，所以这里可以直接使用。当并发特别大时可以考虑细化该锁
     *
     * @param params 请求参数
     * @return /
     */
    private synchronized List<FundPositionBaseInfo> loadFundPositionBaseInfo(P params) {
        PositionLoad loadInstance = this.factory.getPositionLoadInstance(params.getFundId());
        return loadInstance.getFundPositionBaseInfo(params);
    }

    /**
     * 同步的获取基金的持仓信息，获取后放入缓存
     * synchronized 关键字被优化过，所以这里可以直接使用。当并发特别大时可以考虑细化该锁
     *
     * @param params 请求参数
     * @return /
     */
    private synchronized List<FundPositionDetail> loadPositionData(P params) {
        String key = RedisConst.POSITION_DETAIL_KEY;
        String item = params.getFundId();
        Object hget = this.cacheGateway.hget(key, item);
        if (hget != null) {
            return this.convertList(hget, FundPositionDetail.class);
        }
        PositionLoad loadInstance = this.factory.getPositionLoadInstance(params.getFundId());
        List<FundPositionDetail> positionInfoList = loadInstance.loadPositionDetail(params);
        SaveCacheEvent<List<FundPositionDetail>> event = new SaveCacheEvent<>(key, positionInfoList,
                t -> this.cacheGateway.hset(key, item, t, 1, TimeUnit.DAYS));
        this.applicationContext.publishEvent(event);
        return positionInfoList;
    }
}
