package com.smppw.analysis.domain.manager.position;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.position.*;
import com.smppw.analysis.domain.dao.PubliclyFundPositionDao;
import com.smppw.analysis.domain.dataobject.FundPositionBaseInfoDO;
import com.smppw.analysis.domain.dataobject.FundPositionDetailDO;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.constants.SecType;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @author wangzaijun
 * @date 2023/6/7 9:52
 * @description 公募基金持仓数据获取逻辑
 */
@Component(SecType.PUBLICLY_OFFERED_FUNDS)
public class PubliclyFundPositionLoad extends AbstractPositionLoad {
    private static final Map<Integer, AssetCategoryEnum> ASSET_TYPE_MAPPER = MapUtil.newHashMap();

    static {
        ASSET_TYPE_MAPPER.put(0, AssetCategoryEnum.STOCK);
        ASSET_TYPE_MAPPER.put(1, AssetCategoryEnum.BOND);
        ASSET_TYPE_MAPPER.put(3, AssetCategoryEnum.FUTURE);
        ASSET_TYPE_MAPPER.put(6, AssetCategoryEnum.FUND); // 公募
        ASSET_TYPE_MAPPER.put(7, AssetCategoryEnum.FUND); // 私募
    }

    private final PubliclyFundPositionDao publiclyFundPositionDao;

    public PubliclyFundPositionLoad(PubliclyFundPositionDao publiclyFundPositionDao) {
        this.publiclyFundPositionDao = publiclyFundPositionDao;
    }

    @Override
    public <P extends BaseParams> List<FundPositionBaseInfo> getFundPositionBaseInfo(P params) {
        List<FundPositionBaseInfoDO> doList = this.publiclyFundPositionDao.fundPositionBaseInfos(params.getFundId(),
                params.getStartDate(), params.getEndDate());
        List<FundPositionBaseInfo> resultList = ListUtil.list(false);
        for (FundPositionBaseInfoDO temp : doList) {
            if (temp == null || temp.getReportDate() == null) {
                continue;
            }
            String date = DateUtil.formatDate(temp.getReportDate());
            FundPositionBaseInfo info = new FundPositionBaseInfo();
            info.setFundId(temp.getFundId());
            info.setDate(date);
            info.setTotalAsset(temp.getTotalAsset());
            info.setAssetNv(temp.getAssetNv());
            resultList.add(info);
        }
        return resultList;
    }

    @Override
    public <P extends BaseParams> List<FundPositionDetail> loadPositionDetail(P params) {
        List<FundPositionDetailDO> dataList = this.publiclyFundPositionDao.positionDetailList(params.getFundId());
        List<FundPositionDetail> resultList = ListUtil.list(false);
        for (FundPositionDetailDO temp : dataList) {
            if (temp == null || temp.getValuationDate() == null) {
                continue;
            }
            // 公募基金没有成本、估值增值、增值比例和停牌信息
            FundPositionDetail detail = new FundPositionDetail();
            detail.setFundId(temp.getFundId());
            detail.setFundType(SecType.PUBLICLY_OFFERED_FUNDS);
            detail.setDate(DateUtil.formatDate(temp.getValuationDate()));
            String secCode = temp.getSecuritiesCode();
            AssetCategoryEnum categoryEnum = ASSET_TYPE_MAPPER.get(temp.getSecType());
            ValueLabelVO ref = new ValueLabelVO(secCode, temp.getSecuritiesName());
            if (categoryEnum == AssetCategoryEnum.FUTURE && StrUtil.containsAny(secCode, PositionConstants.BOND_BREED_ARR)) {
                // 公募基金国债期货名称特殊处理
                ref = new ValueLabelVO(secCode, secCode);
            }
            detail.setRef(ref);
            detail.setAsset(AssetCategoryEnum.buildValueLabelByAsset(categoryEnum));
            detail.setMarketValue(temp.getMarketValue());
            detail.setRatioNv(temp.getMarketValueRatio());
            detail.setNumber(temp.getSecuritiesAmount());
            detail.setSharesNature(temp.getSharesNature());
            detail.setSecId(temp.getSecId());
            resultList.add(detail);
        }
        return resultList;
    }
}
