package com.smppw.analysis.domain.manager.position.future;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.IterUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.position.AssetCategoryEnum;
import com.smppw.analysis.domain.dto.position.FundPositionBaseInfo;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.dto.position.PositionConstants;
import com.smppw.analysis.domain.dto.position.future.FutureDailyRet;
import com.smppw.analysis.domain.dto.position.future.MarginalRiskContribution;
import com.smppw.analysis.domain.dto.position.future.MarginalRiskContributionParams;
import com.smppw.analysis.domain.dto.position.future.MarginalRiskContributionVO;
import com.smppw.analysis.domain.manager.position.AbstractBizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerConstants;
import com.smppw.analysis.domain.manager.position.PositionLoadFactory;
import com.smppw.analysis.domain.dataobject.FutureDailyPriceDO;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.analysis.infrastructure.exception.APIException;
import com.smppw.analysis.domain.mapper.BaseUnderlyingMapper;
import com.smppw.analysis.domain.mapper.IndexesTradeDateDoMapper;
import com.smppw.common.pojo.FundFuturesOption;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.utils.BigDecimalUtils;
import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.stat.correlation.Covariance;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/19 11:23
 * @description 私募基金期货边际风险贡献
 */
@Component(BizHandlerConstants.MARGINAL_RISK_CONTRIBUTION)
public class MarginalRiskContributionBizHandler extends AbstractBizHandler<MarginalRiskContributionParams, List<MarginalRiskContributionVO>> {
    private final BaseUnderlyingMapper baseUnderlyingMapper;
    private final IndexesTradeDateDoMapper indexesTradeDateDoMapper;

    public MarginalRiskContributionBizHandler(AnalysisProperty property, CacheFactory cacheFactory,
                                              BaseUnderlyingMapper baseUnderlyingMapper,
                                              PositionLoadFactory factory, IndexesTradeDateDoMapper indexesTradeDateDoMapper) {
        super(property, cacheFactory, factory);
        this.baseUnderlyingMapper = baseUnderlyingMapper;
        this.indexesTradeDateDoMapper = indexesTradeDateDoMapper;
    }

    @Override
    protected List<MarginalRiskContributionVO> afterLoadData(MarginalRiskContributionParams params, List<FundPositionBaseInfo> baseInfos, List<FundPositionDetail> positionInfos) {
        if (logger.isDebugEnabled()) {
            logger.debug("持仓数据取数耗时：" + (System.currentTimeMillis() - this.current) + "ms");
        }
        String fundType = positionInfos.get(0).getFundType();
        if (!PositionConstants.PRIVATELY_TYPES.contains(fundType)) {
            throw new APIException(StrUtil.format("该产品【{}】不支持期货边际风险贡献功能！", params.getFundId()));
        }
        List<MarginalRiskContributionVO> resultList = ListUtil.list(false);
        // 得到期货持仓信息，必须是期货，不能包括期权
        positionInfos = positionInfos.stream().filter(e -> e.getLevel() != null && e.getLevel() >= 4).filter(e -> e.getAsset() != null)
                .filter(e -> PositionConstants.FUTURE_SECOND_TYPES.contains(e.getSubType()))
                .filter(e -> e.getNumber() != null && e.getNumber().compareTo(BigDecimal.ZERO) > 0)
                .filter(e -> AssetCategoryEnum.FUTURE.name().equals(e.getAsset().getValue())).collect(Collectors.toList());
        if (CollUtil.isEmpty(positionInfos)) {
            return resultList;
        }
        List<ValueLabelVO> breedNameList = this.baseUnderlyingMapper.getOptionTypeList();
        // 区分金融期货和商品期货，单独取数
        Map<ValueLabelVO, List<String>> financeRefMap = MapUtil.newHashMap();
        Map<ValueLabelVO, List<String>> commodityRefMap = MapUtil.newHashMap();
        this.distRef(breedNameList, positionInfos, financeRefMap, commodityRefMap);
        // 按日期分组
        Map<String, List<FundPositionDetail>> positionMap = positionInfos.stream().collect(Collectors.groupingBy(FundPositionDetail::getDate));
        Map<String, List<FutureDailyRet>> dateRetMap = MapUtil.newHashMap();
        positionMap.forEach((k, v) -> {
            List<String> dateList = this.getRetDates(k);
            String firstDate = dateList.stream().min(Comparator.naturalOrder()).orElse(k);
            // 所有品种的近31个交易日日收益率
            List<FutureDailyRet> dataList = this.getFutureDailyRets(k, dateList, firstDate, financeRefMap, commodityRefMap);
            dateRetMap.put(k, dataList);
        });
        if (logger.isDebugEnabled()) {
            logger.debug("取数耗时：" + (System.currentTimeMillis() - this.current) + "ms");
        }
        positionMap.forEach((k, v) -> {
            List<String> dateList = getRetDates(k);
            // 所有品种的近31个交易日日收益率
            List<FutureDailyRet> dataList = dateRetMap.get(k);
            // 得到计算风险贡献的数据，计算风险贡献
            List<ValueLabelVO> breeds = ListUtil.list(true);
            List<Double> weightList = ListUtil.list(true);
            double[][] retsMatrix = new double[breedNameList.size()][dateList.size() - 1];
            this.getMatrixData(breedNameList, v, dateList, financeRefMap, commodityRefMap, dataList, breeds, weightList, retsMatrix);
            if (CollUtil.isNotEmpty(weightList)) {
                MarginalRiskContributionVO vo = this.calcRiskContAndBuildVo(k, breeds, weightList, retsMatrix);
                resultList.add(vo);
            }
        });
        // 按日期升序
        resultList.sort(Comparator.comparing(MarginalRiskContributionVO::getDate));
        return resultList;
    }

    private List<String> getRetDates(String k) {
        String startDate = DateUtil.formatDate(DateUtil.offsetDay(DateUtil.parseDate(k), -90));
        return this.indexesTradeDateDoMapper.listTradeDates(startDate, k).stream()
                .sorted(Comparator.reverseOrder()).limit(31).sorted().collect(Collectors.toList());
    }

    private MarginalRiskContributionVO calcRiskContAndBuildVo(String k, List<ValueLabelVO> breeds, List<Double> weightList, double[][] retsMatrix) {
        int size = weightList.size();
        double[][] covList = this.getCovByRet(retsMatrix);
        double[] realWeightList = new double[size];
        Double totalW = weightList.stream().reduce(Double::sum).orElse(0d);
        for (int i = 0; i < size; i++) {
            Double weight = weightList.get(i);
            realWeightList[i] = totalW.compareTo(0d) != 0 ? weight / totalW : 1.0d / size;
        }
        double std = this.getStd(covList, realWeightList);
        double[] riskContList = this.getBreedRiskContribution(covList, realWeightList);

        List<MarginalRiskContribution> breedRiskContList = ListUtil.list(false);
        for (int i = 0; i < breeds.size(); i++) {
            ValueLabelVO breed = breeds.get(i);
            String breedValue = breed.getValue();
            ValueLabelVO asset;
            if (PositionConstants.BOND_BREEDS.contains(breedValue)) {
                asset = new ValueLabelVO(FundFuturesOption.Bond.getNameEnDesc(), FundFuturesOption.Bond.getNameDesc());
            } else if (PositionConstants.STOCK_INDEX_BREEDS.contains(breedValue)) {
                asset = new ValueLabelVO(FundFuturesOption.StockIndex.getNameEnDesc(), FundFuturesOption.StockIndex.getNameDesc());
            } else {
                asset = new ValueLabelVO(FundFuturesOption.Product.getNameEnDesc(), FundFuturesOption.Product.getNameDesc());
            }
            MarginalRiskContribution riskCont = new MarginalRiskContribution();
            riskCont.setAsset(asset);
            riskCont.setRef(new ValueLabelVO(breedValue, breed.getLabel()));
            riskCont.setMarketValue(null);
            riskCont.setRatio(BigDecimalUtils.toBigDecimal(weightList.get(i)));
            riskCont.setRiskCont(BigDecimalUtils.toBigDecimal(riskContList[i] / std));
            breedRiskContList.add(riskCont);
        }
        // 按贡献度降序
        breedRiskContList.sort((o1, o2) -> o2.getRiskCont().compareTo(o1.getRiskCont()));
        MarginalRiskContributionVO vo = new MarginalRiskContributionVO();
        vo.setDate(k);
        vo.setRiskContList(breedRiskContList);
        return vo;
    }

    private void getMatrixData(List<ValueLabelVO> breedNameList, List<FundPositionDetail> v, List<String> dateList,
                               Map<ValueLabelVO, List<String>> financeRefMap, Map<ValueLabelVO, List<String>> commodityRefMap,
                               List<FutureDailyRet> dataList, List<ValueLabelVO> breeds, List<Double> weightList, double[][] retsMatrix) {
        for (int i = 0; i < breedNameList.size(); i++) {
            ValueLabelVO breed = breedNameList.get(i);
            List<String> refIds = ListUtil.list(false);
            List<String> finance = financeRefMap.getOrDefault(breed, ListUtil.empty());
            if (CollUtil.isNotEmpty(finance)) {
                refIds.addAll(finance);
            }
            List<String> commdiity = commodityRefMap.getOrDefault(breed, ListUtil.empty());
            if (CollUtil.isNotEmpty(commdiity)) {
                refIds.addAll(commdiity);
            }
            List<String> ids = refIds.stream().filter(Objects::nonNull).collect(Collectors.toList());
            if (CollUtil.isEmpty(ids)) {
                continue;
            }
            List<FundPositionDetail> details = v.stream().filter(e -> ids.contains(e.getRef().getValue()) || ids.contains(e.getSecId())).collect(Collectors.toList());
            // 品种总市值比之和
            Double weight = details.stream().map(FundPositionDetail::getRatioNv).map(BigDecimal::doubleValue).reduce(Double::sum).orElse(0d);
            if (weight.compareTo(0d) == 0) {
                continue;
            }
            List<FutureDailyRet> reteList = dataList.stream().filter(e -> ids.contains(e.getRefId())).collect(Collectors.toList());
            int size = dateList.size();
            double[] rets = new double[size - 1];
            for (int j = 1; j < size; j++) {
                String date = dateList.get(j);
                double ret = 0d;
                for (String refId : ids) {
                    Double refRet = reteList.stream().filter(e -> refId.equals(e.getRefId())).filter(e -> date.equals(e.getPriceDate()))
                            .findFirst().map(FutureDailyRet::getRet).orElse(0d);
                    double refWeight = details.stream().filter(e -> refId.equals(e.getSecId()) || refId.equals(e.getRef().getValue()))
                            .map(FundPositionDetail::getRatioNv).map(BigDecimal::doubleValue).reduce(Double::sum).orElse(0d);
                    double v1 = refWeight / weight;
                    ret += refRet * v1;
                }
                rets[j] = ret;
            }
            if (IterUtil.isAllNull(Collections.singleton(rets)) || rets.length != 30) {
                continue;
            }
            weightList.add(weight);
            retsMatrix[i] = rets;
            breeds.add(breed);
        }
    }

    private List<FutureDailyRet> getFutureDailyRets(String k, List<String> dateList, String firstDate, Map<ValueLabelVO, List<String>> financeRefMap, Map<ValueLabelVO, List<String>> commodityRefMap) {
        List<FutureDailyRet> dataList = ListUtil.list(false);
        List<String> financeRefIds = financeRefMap.values().stream().flatMap(Collection::stream).distinct().collect(Collectors.toList());
        List<String> commodityRefIds = commodityRefMap.values().stream().flatMap(Collection::stream).distinct().collect(Collectors.toList());
        if (CollUtil.isNotEmpty(financeRefIds)) {
            // 金融期货直接取张跌幅
            List<FutureDailyPriceDO> tempList = this.baseUnderlyingMapper.getFinanceSecClosePrice(financeRefIds, firstDate, k);
            List<FutureDailyRet> collect = tempList.stream().map(e -> {
                Double ret = e.getClosePriceRet();
                if (ret != null) {
                    ret = ret / 100d;
                }
                return new FutureDailyRet(e.getRefId(), e.getPriceDate(), ret);
            }).collect(Collectors.toList());
            if (CollUtil.isNotEmpty(collect)) {
                dataList.addAll(collect);
            }
        }
        if (CollUtil.isNotEmpty(commodityRefIds)) {
            // 商品期货只能取收盘价实时计算
            List<FutureDailyPriceDO> tempList = this.baseUnderlyingMapper.getProductSecClosePrice(commodityRefIds, firstDate, k);
            if (CollUtil.isNotEmpty(tempList)) {
                Map<String, List<FutureDailyPriceDO>> collect = tempList.stream().collect(Collectors.groupingBy(FutureDailyPriceDO::getRefId));
                collect.forEach((refId, l) -> {
                    for (int i = 0; i < dataList.size(); i++) {
                        String date = dateList.get(i);
                        String prevDate = i == 0 ? dateList.get(i) : dateList.get(i - 1);
                        FutureDailyPriceDO temp = l.stream().filter(e -> date.equals(e.getPriceDate())).findFirst().orElse(null);
                        FutureDailyPriceDO temp1 = l.stream().filter(e -> prevDate.equals(e.getPriceDate())).findFirst().orElse(null);
                        double ret = 0;
                        if (i != 0 && temp != null && temp1 != null) {
                            ret = temp.getClosePrice() / temp1.getClosePrice() - 1;
                        }
                        dataList.add(new FutureDailyRet(refId, date, ret));
                    }
                });
            }
        }
        return dataList;
    }

    private void distRef(List<ValueLabelVO> breedNameList, List<FundPositionDetail> v, Map<ValueLabelVO, List<String>> financeRefMap, Map<ValueLabelVO, List<String>> commodityRefMap) {
        // 遍历品种获取各品种下的合约
        breedNameList.forEach(breed -> {
            String breedValue = breed.getValue();
            List<FundPositionDetail> details = v.stream().filter(e -> {
                String futureCode = e.getRef().getValue();
                String tradingCode = futureCode.substring(0, 2);
                if (!tradingCode.matches("[A-Za-z]+")) {
                    tradingCode = tradingCode.substring(0, 1);
                }
                return breedValue.equals(tradingCode);
            }).collect(Collectors.toList());
            if (PositionConstants.STOCK_INDEX_BREEDS.contains(breedValue) || PositionConstants.BOND_BREEDS.contains(breedValue)) {
                List<String> refIds = details.stream().map(e -> e.getRef().getValue()).distinct().collect(Collectors.toList());
                financeRefMap.put(breed, refIds);
            } else {
                List<String> refIds = details.stream().map(FundPositionDetail::getSecId).distinct().collect(Collectors.toList());
                commodityRefMap.put(breed, refIds);
            }
        });
    }

    private double[] getBreedRiskContribution(double[][] retsArr, double[] weightArr) {
        RealMatrix retsArrMatrix = new Array2DRowRealMatrix(retsArr);
        RealMatrix weightArrMatrix = new Array2DRowRealMatrix(weightArr);
        RealMatrix matrix = weightArrMatrix.transpose().multiply(retsArrMatrix);
        return matrix.getData()[0];
    }

    private double getStd(double[][] retsMatrix, double[] weight) {
        RealMatrix retsArrMatrix = new Array2DRowRealMatrix(retsMatrix);
        RealMatrix weightArrMatrix = new Array2DRowRealMatrix(weight);
        RealMatrix matrix = weightArrMatrix.transpose().multiply(retsArrMatrix).multiply(weightArrMatrix);
        return Math.sqrt(matrix.getData()[0][0]);
    }

    private double[][] getCovByRet(double[][] retsArr) {
        RealMatrix retsArrMatrix = new Array2DRowRealMatrix(retsArr);
        RealMatrix covMatrix = new Covariance(retsArrMatrix, true).getCovarianceMatrix();
        return covMatrix.getData();
    }
}
