package com.smppw.analysis.domain.manager.position.stock;

import cn.hutool.core.collection.ListUtil;
import com.smppw.analysis.domain.dto.position.AssetCategoryEnum;
import com.smppw.analysis.domain.dto.position.FundPositionBaseInfo;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.dto.position.RefMarketValueRatio;
import com.smppw.analysis.domain.dto.position.stock.ConcentrationParams;
import com.smppw.analysis.domain.dto.position.stock.ConcentrationVO;
import com.smppw.analysis.domain.manager.position.AbstractNonSynthesizeBizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerConstants;
import com.smppw.analysis.domain.manager.position.PositionLoadFactory;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.constants.SecType;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/8 14:07
 * @description 股票集中度业务数据转换
 */
@Component(BizHandlerConstants.STOCK_CONCENTRATION)
public class ConcentrationBizHandler extends AbstractNonSynthesizeBizHandler<ConcentrationParams, List<ConcentrationVO>> {

    public ConcentrationBizHandler(AnalysisProperty property, CacheFactory cacheFactory, PositionLoadFactory factory) {
        super(property, cacheFactory, factory);
    }

    @Override
    protected List<ConcentrationVO> afterNonSynthesizeLoadData(ConcentrationParams params, List<FundPositionBaseInfo> baseInfos, List<FundPositionDetail> positionInfos) {
        positionInfos = positionInfos.stream().filter(e -> SecType.PUBLICLY_OFFERED_FUNDS.equals(e.getFundType()) || (e.getLevel() != null && e.getLevel() >= 4))
                .filter(e -> AssetCategoryEnum.STOCK.name().equals(e.getAsset().getValue())).collect(Collectors.toList());
        List<ConcentrationVO> resultList = ListUtil.list(false);
        // 按日期分组处理
        Map<String, List<FundPositionDetail>> collect = positionInfos.stream().collect(Collectors.groupingBy(FundPositionDetail::getDate));
        collect.forEach((k, v) -> {
            // 基金在当前日期的净资产值
            BigDecimal bigDecimal = baseInfos.stream().filter(e -> k.equals(e.getDate())).findFirst().map(FundPositionBaseInfo::getAssetNv).orElse(null);
            if (bigDecimal != null && bigDecimal.abs().compareTo(BigDecimal.ZERO) > 0) {
                // 按成分分组，多空轧差
                Map<ValueLabelVO, List<BigDecimal>> listMap = v.stream().collect(Collectors.groupingBy(FundPositionDetail::getRef,
                        Collectors.mapping(FundPositionDetail::getMarketValue, Collectors.toList())));
                List<RefMarketValueRatio> position = ListUtil.list(false);
                listMap.forEach((r, m) -> {
                    BigDecimal mv = m.stream().reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
                    RefMarketValueRatio mvr = new RefMarketValueRatio(r, mv, mv.divide(bigDecimal, 10, RoundingMode.HALF_UP));
                    position.add(mvr);
                });
                // 按占比降序
                position.sort((o1, o2) -> o2.getRatio().compareTo(o1.getRatio()));
                ConcentrationVO vo = new ConcentrationVO();
                vo.setDate(k);
                vo.setAssetNv(bigDecimal);
                vo.setPosition(position);
                resultList.add(vo);
            }
        });
        // 按日期升序
        resultList.sort(Comparator.comparing(ConcentrationVO::getDate));
        return resultList;
    }
}