package com.smppw.analysis.domain.manager.position.stock;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.position.CategoryConstraint;
import com.smppw.analysis.domain.dto.position.CompositionCategory;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.dto.position.PositionLiquidityEnum;
import com.smppw.analysis.domain.dto.position.stock.StockAllocationParams;
import com.smppw.analysis.domain.dto.position.stock.StockAllocationVO;
import com.smppw.analysis.domain.manager.position.AbstractAnalysisBizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerConstants;
import com.smppw.analysis.domain.manager.position.PositionLoadFactory;
import com.smppw.analysis.domain.dataobject.IndexSecWeightInfoDO;
import com.smppw.analysis.domain.dataobject.SecLiquidityInfoDO;
import com.smppw.analysis.domain.dataobject.SecTurnoverInfoDO;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.domain.dao.PositionAnalysisDao;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.constants.SecType;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/9 14:20
 * @description 流动性分析
 */
@Component(BizHandlerConstants.LIQUIDITY_ALLOCATION)
public class NewLiquidityAllocationBizHandler extends AbstractAnalysisBizHandler<StockAllocationParams, List<StockAllocationVO>> {


    public NewLiquidityAllocationBizHandler(AnalysisProperty property, CacheFactory cacheFactory,
                                            PositionLoadFactory factory, PositionAnalysisDao analysisService) {
        super(property, cacheFactory, factory, analysisService);
    }

    @Override
    protected List<StockAllocationVO> afterAnalysisLoadData(String benchmarkId, List<FundPositionDetail> positionDetails,
                                                            Map<String, List<IndexSecWeightInfoDO>> indexWeightMap) {
        if (logger.isDebugEnabled()) {
            logger.debug("流动性分析配置 数据准备耗时：" + (System.currentTimeMillis() - this.current) + "ms");
        }
        List<ValueLabelVO> allCategoryList = Arrays.stream(PositionLiquidityEnum.values())
                .map(e -> new ValueLabelVO(e.name(), e.getDesc())).collect(Collectors.toList());
        // 日期分组
        Map<String, List<FundPositionDetail>> positionMap = positionDetails.stream().collect(Collectors.groupingBy(FundPositionDetail::getDate));
        // 日期升序
        List<String> dateList = positionMap.keySet().stream().sorted().collect(Collectors.toList());
        List<String> realDates = dateList;
        String fundType = positionDetails.get(0).getFundType();
        if (SecType.PUBLICLY_OFFERED_FUNDS.equals(fundType)) {
            realDates = dateList.stream().map(this::getRealDate).distinct().collect(Collectors.toList());
        }
        // 流动性标准
        List<SecLiquidityInfoDO> secLiquidityInfos = this.analysisService.listSecLiquidityInfo(realDates);
        // 5日换手率
        List<SecTurnoverInfoDO> secTurnoverInfos = this.analysisService.listSecTurnoverInfo(realDates);

        Map<String, Map<ValueLabelVO, List<Map<String, Object>>>> secStyleMap = MapUtil.newHashMap();
        Map<String, Map<ValueLabelVO, List<Map<String, Object>>>> indexSecStyleMap = MapUtil.newHashMap();
        // 数据转换
        for (String date : dateList) {
            String realDate = SecType.PUBLICLY_OFFERED_FUNDS.equals(fundType) ? this.getRealDate(date) : date;
            List<SecTurnoverInfoDO> turnoverInfos = secTurnoverInfos.stream().filter(e -> realDate.equals(e.getTradingDay())).collect(Collectors.toList());
            SecLiquidityInfoDO liquidityInfo = secLiquidityInfos.stream().filter(e -> realDate.equals(e.getTradingDay())).findFirst().orElse(null);
            List<CompositionCategory> categoryList = ListUtil.list(false);
            turnoverInfos.forEach(e -> {
                Double secRate = e.getTurnoverValue();
                PositionLiquidityEnum liquidity = PositionLiquidityEnum.OTHER;
                if (secRate != null && liquidityInfo != null) {
                    if (liquidityInfo.getLowestOne() != null && secRate >= liquidityInfo.getLowestOne()) {
                        liquidity = PositionLiquidityEnum.HIGH;
                    } else if (liquidityInfo.getLowestTwo() != null && secRate >= liquidityInfo.getLowestTwo()) {
                        liquidity = PositionLiquidityEnum.HIGHER;
                    } else if (liquidityInfo.getLowestThree() != null && secRate >= liquidityInfo.getLowestThree()) {
                        liquidity = PositionLiquidityEnum.SAME_AS;
                    } else if (liquidityInfo.getLowestFour() != null && secRate >= liquidityInfo.getLowestFour()) {
                        liquidity = PositionLiquidityEnum.LOWER;
                    } else {
                        liquidity = PositionLiquidityEnum.LOW;
                    }
                }
                CompositionCategory category = CompositionCategory.builder().secCode(e.getSecCode())
                        .category(new ValueLabelVO(liquidity.name(), liquidity.getDesc())).build();
                categoryList.add(category);
            });

            // 映射风格（行业、风格或流动性）
            List<Map<String, Object>> list = positionMap.get(date).stream().map(e -> {
                Map<String, Object> temp = MapUtil.newHashMap();
                // 找到当前成分所在风格（行业、风格或流动性），-1 表示其他
                ValueLabelVO style = categoryList.stream().filter(o -> o.getSecCode().equals(e.getRef().getValue()))
                        .findFirst().map(CompositionCategory::getCategory).orElse(OTHER);
                temp.put("ref", e.getRef().getValue());
                temp.put("style", style);
                temp.put("sharesNature", e.getSharesNature());
                temp.put("marketValue", e.getMarketValue());
                return temp;
            }).collect(Collectors.toList());
            Map<ValueLabelVO, List<Map<String, Object>>> styleMap = list.stream()
                    .collect(Collectors.groupingBy(e -> MapUtil.get(e, "style", ValueLabelVO.class)));
            secStyleMap.put(date, styleMap);
            List<Map<String, Object>> indexList = indexWeightMap.get(date).stream().map(e -> {
                Map<String, Object> temp = MapUtil.newHashMap();
                // 找到当前成分所在风格（行业、风格或流动性），-1 表示其他
                ValueLabelVO style = categoryList.stream().filter(o -> o.getSecCode().equals(e.getSecCode()))
                        .findFirst().map(CompositionCategory::getCategory).orElse(OTHER);
                temp.put("ref", e.getSecCode());
                temp.put("style", style);
                temp.put("weight", e.getWeight());
                return temp;
            }).collect(Collectors.toList());
            Map<ValueLabelVO, List<Map<String, Object>>> indexStyleMap = indexList.stream()
                    .collect(Collectors.groupingBy(e -> MapUtil.get(e, "style", ValueLabelVO.class)));
            indexSecStyleMap.put(date, indexStyleMap);
        }
        if (this.logger.isDebugEnabled()) {
            this.logger.debug(StrUtil.format("所有数据准备：{} ms", System.currentTimeMillis() - this.current));
        }

        List<StockAllocationVO> resultList = ListUtil.list(false);
        positionMap.forEach((k, v) -> {
            // 股票市值
            BigDecimal total = v.stream().map(FundPositionDetail::getMarketValue).filter(Objects::nonNull)
                    .reduce(BigDecimal::add).map(BigDecimal::abs).orElse(null);
            List<CategoryConstraint> constraintList = ListUtil.list(false);
            Map<ValueLabelVO, List<Map<String, Object>>> styleMap = secStyleMap.get(k);
            Map<ValueLabelVO, List<Map<String, Object>>> indexStyleMap = indexSecStyleMap.get(k);
            for (ValueLabelVO category : allCategoryList) {
                // 找到同风格的基金成分
                List<Map<String, Object>> l = ListUtil.empty();
                if (styleMap != null && styleMap.get(category) != null) {
                    l = styleMap.get(category);
                }
                CategoryConstraint constraint = this.calcCategoryWeight(total, indexStyleMap, category, l);
                constraintList.add(constraint);
            }
            // 按多空降序
            constraintList.sort(((o1, o2) -> o2.getPupil().compareTo(o1.getPupil())));
            StockAllocationVO vo = new StockAllocationVO();
            vo.setDate(k);
            vo.setIndustries(constraintList);
            resultList.add(vo);
        });
        // 按日期升序
        resultList.sort(Comparator.comparing(StockAllocationVO::getDate));
        return resultList;
    }

    /**
     * 找当前估值日期最近的交易日（往前找）
     *
     * @param date 估值日期
     * @return /
     */
    private String getRealDate(String date) {
        String s = this.analysisService.getLatestTradeDate(date);
        return s == null ? date : s;
    }
}
