package com.smppw.analysis.domain.manager.position.stock;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.dto.position.stock.StockPerformanceAttributionParams;
import com.smppw.analysis.domain.dto.position.stock.StockPerformanceAttributionVO;
import com.smppw.analysis.domain.manager.position.AbstractAnalysisBizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerConstants;
import com.smppw.analysis.domain.manager.position.PositionLoadFactory;
import com.smppw.analysis.domain.dataobject.IndexSecWeightInfoDO;
import com.smppw.analysis.domain.dataobject.SecClosePriceDO;
import com.smppw.analysis.domain.event.SaveCacheEvent;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.domain.dao.PositionAnalysisDao;
import com.smppw.analysis.domain.service.NavService;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.analysis.infrastructure.consts.RedisConst;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.common.pojo.dto.DateValue;
import com.smppw.common.pojo.enums.NavType;
import com.smppw.constants.SecType;
import com.smppw.utils.BigDecimalUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/13 13:38
 * @description 股票业绩归因
 */
@Component(BizHandlerConstants.STOCK_PERFORMANCE_ATTRIBUTION)
public class StockPerformanceAttributionBizHandler extends AbstractAnalysisBizHandler<StockPerformanceAttributionParams, List<StockPerformanceAttributionVO>> {
    private final NavService navService;

    public StockPerformanceAttributionBizHandler(AnalysisProperty property, CacheFactory cacheFactory,
                                                 PositionLoadFactory factory, PositionAnalysisDao analysisService, NavService navService) {
        super(property, cacheFactory, factory, analysisService);
        this.navService = navService;
    }

    @Override
    protected List<StockPerformanceAttributionVO> afterAnalysisLoadData(String benchmarkId, List<FundPositionDetail> positionDetails,
                                                                        Map<String, List<IndexSecWeightInfoDO>> indexWeightMap) {
        String fundType = positionDetails.get(0).getFundType();
        Map<ValueLabelVO, List<String>> industrySecMap = this.getIndustrySecMap();
        Map<String, List<FundPositionDetail>> positionMap = positionDetails.stream().collect(Collectors.groupingBy(FundPositionDetail::getDate));
        List<String> dateList = positionMap.keySet().stream().sorted().collect(Collectors.toList());
        // 数据获取和转换
        Map<String, Map<String, Double>> dateSecRetMap = MapUtil.newHashMap();
        if (SecType.PUBLICLY_OFFERED_FUNDS.equals(fundType)) {
            List<String> realDates = dateList.stream().map(this::getRealDate).collect(Collectors.toList());
            List<SecClosePriceDO> dataList = this.analysisService.listSecClosePrice(realDates);
            List<SecClosePriceDO> tempList = this.analysisService.listSciTechSecClosePrice(realDates);
            if (CollUtil.isNotEmpty(tempList)) {
                dataList.addAll(tempList);
            }
            Map<String, Map<String, Double>> collect = dataList.stream().collect(Collectors.groupingBy(SecClosePriceDO::getTradingDay,
                    Collectors.toMap(SecClosePriceDO::getSecCode, e -> e.getRet6m() == null ? 0d : e.getRet6m() / 100d)));
            dateSecRetMap.putAll(collect);
        } else {
            // 全量数据获取
            List<SecClosePriceDO> multiSecPrices = this.getMultiSecPrices(dateList);
            List<Map<String, CompletableFuture<Map<String, Double>>>> futures = ListUtil.list(false);
            for (int i = 0; i < dateList.size(); i++) {
                String prevDate = i == 0 ? dateList.get(0) : dateList.get(i - 1);
                String date = dateList.get(i);
                // 获取所有股票代码
                List<FundPositionDetail> fundPositionDetails = positionMap.get(date);
                List<IndexSecWeightInfoDO> secWeightList = indexWeightMap.getOrDefault(date, ListUtil.empty());
                List<String> secCodes = fundPositionDetails.stream().map(e -> e.getRef().getValue()).distinct().collect(Collectors.toList());
                if (CollUtil.isNotEmpty(secWeightList)) {
                    List<String> indexSecCodes = secWeightList.stream().map(IndexSecWeightInfoDO::getSecCode).distinct().collect(Collectors.toList());
                    CollUtil.addAllIfNotContains(secCodes, indexSecCodes);
                }
                // 对于计算密集型，可以用默认的线程池（计算每个股票的收益率）
                CompletableFuture<Map<String, Double>> future = CompletableFuture.supplyAsync(
                        () -> this.calcSecDateRet(prevDate, date, secCodes, multiSecPrices), DEFAULT_EXECUTOR);
                futures.add(MapUtil.<String, CompletableFuture<Map<String, Double>>>builder().put(date, future).build());
            }
            for (Map<String, CompletableFuture<Map<String, Double>>> future : futures) {
                future.forEach((k, v) -> dateSecRetMap.put(k, v.join()));
            }
        }
        if (logger.isDebugEnabled()) {
            logger.debug("业绩归因数据处理耗时：" + (System.currentTimeMillis() - this.current) + "ms");
        }
        // 业务处理
        List<CompletableFuture<StockPerformanceAttributionVO>> futures = ListUtil.list(false);
        for (int i = 0; i < dateList.size(); i++) {
            String prevDate = i == 0 ? dateList.get(0) : dateList.get(i - 1);
            String date = dateList.get(i);
            List<FundPositionDetail> fundPositionDetails = positionMap.get(date);
            List<IndexSecWeightInfoDO> secWeightList = indexWeightMap.getOrDefault(date, ListUtil.empty());
            Map<String, Double> secRetMap = dateSecRetMap.getOrDefault(date, MapUtil.empty());
            // 计算业绩归因
            CompletableFuture<StockPerformanceAttributionVO> future = CompletableFuture.supplyAsync(
                    () -> this.calcPerformanceAttribution(date, prevDate, benchmarkId, fundPositionDetails, industrySecMap, secWeightList, secRetMap), DEFAULT_EXECUTOR);
            futures.add(future);
        }
        List<StockPerformanceAttributionVO> resultList = ListUtil.list(false);
        for (CompletableFuture<StockPerformanceAttributionVO> future : futures) {
            resultList.add(future.join());
        }
        resultList.sort(Comparator.comparing(StockPerformanceAttributionVO::getDate));
        return resultList;
    }

    private StockPerformanceAttributionVO calcPerformanceAttribution(String date, String prevDate, String benchmarkId,
                                                                     List<FundPositionDetail> fundPositionDetails, Map<ValueLabelVO, List<String>> industrySecMap,
                                                                     List<IndexSecWeightInfoDO> secWeightList, Map<String, Double> secRetMap) {
        Double totalMv = fundPositionDetails.stream().map(FundPositionDetail::getMarketValue).filter(Objects::nonNull)
                .reduce(BigDecimal::add).map(BigDecimal::abs).map(BigDecimal::doubleValue).orElse(null);
        List<StockPerformanceAttributionVO.Effect> effects = ListUtil.list(true);
        double indexRet = this.getIndexRet(benchmarkId, prevDate, date);
        industrySecMap.forEach((c, l) -> {
            // 计算各行业下的效应
            StockPerformanceAttributionVO.Effect effect = this.calcEffect(c, fundPositionDetails, totalMv, secWeightList, secRetMap, indexRet, l);
            effects.add(effect);
        });
        // 按超额收益降序
        effects.sort(((o1, o2) -> o2.getRatio().compareTo(o1.getRatio())));
        StockPerformanceAttributionVO vo = new StockPerformanceAttributionVO();
        vo.setDate(date);
        vo.setEffect(effects);
        return vo;
    }

    private StockPerformanceAttributionVO.Effect calcEffect(ValueLabelVO category, List<FundPositionDetail> positionDetails, Double totalMv,
                                                            List<IndexSecWeightInfoDO> secWeightList, Map<String, Double> secRetMap, double indexRet, List<String> l) {
        List<IndexSecWeightInfoDO> collect = secWeightList.stream().filter(e -> l.contains(e.getSecCode())).collect(Collectors.toList());
        // 指数成分的行业收益与权重汇总
        double indexWeight = collect.stream().map(IndexSecWeightInfoDO::getWeight)
                .map(BigDecimal::doubleValue).reduce(Double::sum).map(e -> e / 100d).orElse(0d);
        double indexIndustryRet = collect.stream().map(e -> indexWeight * secRetMap.getOrDefault(e.getSecCode(), 0d))
                .reduce(Double::sum).orElse(0d);
        double secWeight = 0d;
        double industryRet = 0d;
        if (totalMv != null && totalMv.compareTo(0d) > 0) {
            // 基金的成分的行业收益与占市值比汇总
            List<ValueLabelVO> collect1 = positionDetails.stream().filter(e -> l.contains(e.getRef().getValue()))
                    .map(e -> new ValueLabelVO(e.getRef().getValue(), e.getMarketValue().toPlainString())).collect(Collectors.toList());
            secWeight = collect1.stream().map(ValueLabelVO::getLabel).map(Double::parseDouble).reduce(Double::sum).map(e -> e / totalMv).orElse(0d);
            industryRet = collect1.stream().map(ValueLabelVO::getValue).map(e -> indexWeight * secRetMap.getOrDefault(e, 0d))
                    .reduce(Double::sum).orElse(0d);
        }
        StockPerformanceAttributionVO.Effect effect = new StockPerformanceAttributionVO.Effect();
        effect.setRef(category);
        effect.setRatio(BigDecimalUtils.toBigDecimal(secWeight));
        // (行业权重 - 基准行业权重) * (基金行业收益 * 基准行业权重 - 基准总收益)
        double allocation = (secWeight - indexWeight) * (indexIndustryRet * indexWeight - indexRet);
        effect.setAllocation(allocation);
        // 行业权重 * (行业收益 * 行业权重 - 基准行业收益 * 基准行业权重)
        double stock = secWeight * (industryRet * secWeight - indexIndustryRet * indexWeight);
        effect.setStock(stock);
        // （组合行业权重-基准行业权重）*基准总收益
        double timing = (secWeight - indexWeight) * indexRet;
        effect.setTiming(timing);
        // (1+持仓组合收益)/(1+基准指数收益)-1 = 配置+选股+择时
        double total = allocation + stock + timing;
        effect.setExactRet(total);
        return effect;
    }

    private double getIndexRet(String benchmarkId, String prevDate, String date) {
        double indexRet = 0d;
        if (!prevDate.equals(date)) {
            // 上一个日期也往前找交易日
            String realDate = this.getRealDate(prevDate);
            Map<String, List<DateValue>> indexNavMap = navService.getMarketIndexNavMap(ListUtil.of(benchmarkId), realDate, date, NavType.CumulativeNav);
            if (MapUtil.isNotEmpty(indexNavMap)) {
                List<DateValue> dateValues = indexNavMap.get(benchmarkId);
                if (CollUtil.isNotEmpty(dateValues)) {
                    indexRet = dateValues.get(dateValues.size() - 1).getValue() / dateValues.get(0).getValue() - 1d;
                }
            }
        }
        return indexRet;
    }

    private Map<String, Double> calcSecDateRet(String prevDate, String date, List<String> secCodes, List<SecClosePriceDO> multiSecPrices) {
        Map<String, Double> secRetMap = MapUtil.newHashMap();
        // 过滤价格
        List<SecClosePriceDO> prices = multiSecPrices.stream().filter(e -> prevDate.compareTo(e.getTradingDay()) <= 0)
                .filter(e -> date.compareTo(e.getTradingDay()) >= 0).collect(Collectors.toList());
        for (String secCode : secCodes) {
            double ret = 0d;
            if (!date.equals(prevDate)) {
                List<SecClosePriceDO> tempList = prices.stream().filter(e -> secCode.equals(e.getSecCode()))
                        .sorted(Comparator.comparing(SecClosePriceDO::getTradingDay)).collect(Collectors.toList());
                if (CollUtil.isNotEmpty(tempList)) {
                    Double prevClose = tempList.get(0).getClosePrice();
                    Double close = tempList.get(tempList.size() - 1).getClosePrice();
                    if (prevClose != null && close != null) {
                        ret = close / prevClose - 1.0d;
                    }
                }
            }
            secRetMap.put(secCode, ret);
        }
        return secRetMap;
    }

    private List<SecClosePriceDO> getMultiSecPrices(List<String> dateList) {
        List<SecClosePriceDO> resultList = ListUtil.list(false);
        if (CollUtil.isEmpty(dateList)) {
            return resultList;
        }
        String firstDate = dateList.get(0);
        String lastDate = dateList.get(dateList.size() - 1);
        String key = RedisConst.POSITION_STOCK_PRICE_KEY;
        String item = firstDate + lastDate;
        Object hget = this.cacheGateway.hget(key, item);
        if (hget != null) {
            resultList = this.convertList(hget, SecClosePriceDO.class);
        } else {
            resultList = this.analysisService.listSecClosePrice(firstDate, lastDate);
            List<SecClosePriceDO> tempList = this.analysisService.listSciTechSecClosePrice(firstDate, lastDate);
            if (CollUtil.isNotEmpty(tempList)) {
                resultList.addAll(tempList);
            }
            SaveCacheEvent<List<SecClosePriceDO>> event = new SaveCacheEvent<>(key, resultList,
                    t -> this.cacheGateway.hset(key, item, t, RedisConst.POSITION_YEAR_TTL, TimeUnit.SECONDS));
            this.applicationContext.publishEvent(event);
        }
        return resultList;
    }

    /**
     * 找当前估值日期最近的交易日（往前找）
     *
     * @param date 估值日期
     * @return /
     */
    private String getRealDate(String date) {
        String s = this.analysisService.getLatestTradeDate(date);
        return s == null ? date : s;
    }
}
