package com.smppw.analysis.domain.manager.position.stock;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.position.CategoryConstraint;
import com.smppw.analysis.domain.dto.position.CompositionCategory;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.dto.position.PositionStyleEnum;
import com.smppw.analysis.domain.dto.position.stock.StockAllocationParams;
import com.smppw.analysis.domain.dto.position.stock.StockAllocationVO;
import com.smppw.analysis.domain.manager.position.AbstractAnalysisBizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerConstants;
import com.smppw.analysis.domain.manager.position.PositionLoadFactory;
import com.smppw.analysis.domain.dataobject.IndexSecWeightInfoDO;
import com.smppw.analysis.domain.dataobject.SecStyleInfoDO;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.domain.dao.PositionAnalysisDao;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.constants.Consts;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/9 14:20
 * @description 风格配置
 */
@Component(BizHandlerConstants.STYLE_ALLOCATION)
public class StyleAllocationBizHandler extends AbstractAnalysisBizHandler<StockAllocationParams, List<StockAllocationVO>> {
    public StyleAllocationBizHandler(AnalysisProperty property, CacheFactory cacheFactory,
                                     PositionLoadFactory factory, PositionAnalysisDao analysisService) {
        super(property, cacheFactory, factory, analysisService);
    }

    @Override
    protected List<StockAllocationVO> afterAnalysisLoadData(String benchmarkId, List<FundPositionDetail> positionDetails,
                                                            Map<String, List<IndexSecWeightInfoDO>> indexWeightMap) {
        if (logger.isDebugEnabled()) {
            logger.debug("风格配置 数据准备耗时：" + (System.currentTimeMillis() - this.current) + "ms");
        }
        // 日期分组
        Map<String, List<FundPositionDetail>> positionMap = positionDetails.stream().collect(Collectors.groupingBy(FundPositionDetail::getDate));
        // 日期升序
        List<String> dateList = positionMap.keySet().stream().sorted().collect(Collectors.toList());
        // 估值日往后找日期，今年用去年的
        List<String> dates = dateList.stream().map(this::getRealDate).distinct().collect(Collectors.toList());
        // 这里不用缓存，缓存会更慢
        List<SecStyleInfoDO> styleInfos = this.analysisService.listSecStyleInfo(dates);
        Map<String, List<SecStyleInfoDO>> styleInfoMap = styleInfos.stream().collect(Collectors.groupingBy(SecStyleInfoDO::getEndDate));

        List<StockAllocationVO> resultList = ListUtil.list(false);
        positionMap.forEach((k, v) -> {
            String realDate = this.getRealDate(k);
            // 股票市值
            BigDecimal total = v.stream().map(FundPositionDetail::getMarketValue).filter(Objects::nonNull)
                    .reduce(BigDecimal::add).map(BigDecimal::abs).orElse(null);
            List<CategoryConstraint> constraintList = ListUtil.list(false);
            List<SecStyleInfoDO> tempList = styleInfoMap.getOrDefault(realDate, ListUtil.empty());
            Map<ValueLabelVO, List<String>> secStyleMap = tempList.stream().map(e -> {
                PositionStyleEnum style = PositionStyleEnum.getStyle(e.getMarket(), e.getBalance());
                return CompositionCategory.builder().secCode(e.getSecCode())
                        .category(style == null ? OTHER : new ValueLabelVO(style.name(), style.getName())).build();
            }).collect(Collectors.groupingBy(CompositionCategory::getCategory, Collectors.mapping(CompositionCategory::getSecCode, Collectors.toList())));
            List<IndexSecWeightInfoDO> indexSecWeightList = indexWeightMap.getOrDefault(k, ListUtil.empty());
            List<String> categoryCodes = ListUtil.list(false);
            secStyleMap.forEach((category, l) -> {
                CategoryConstraint constraint = this.buildCategoryConstraint(v, total, categoryCodes, indexSecWeightList, l);
                constraint.setCategory(category);
                constraintList.add(constraint);
            });
            StockAllocationVO vo = this.getStockAllocationVO(k, v, total, indexSecWeightList, constraintList, categoryCodes);
            resultList.add(vo);
        });
        // 按日期升序
        resultList.sort(Comparator.comparing(StockAllocationVO::getDate));
        return resultList;
    }

    /**
     * 估值日往后找风格日期，今年用去年的
     *
     * @param date 估值日期
     * @return 风格截止日期
     */
    private String getRealDate(String date) {
        Date now = new Date();
        String year = StrUtil.subBefore(date, "-", false);
        String current = DateUtil.format(now, Consts.YEAR_DATE_PATTERN);
        if (current.equals(year)) {
            DateTime offset = DateUtil.offset(now, DateField.YEAR, -1);
            year = DateUtil.format(offset, Consts.YEAR_DATE_PATTERN);
        }
        return year + "-12-31";
    }
}
