package com.smppw.analysis.domain.manager.position.synthesize;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dto.position.*;
import com.smppw.analysis.domain.dto.position.synthesize.AssetAllocationParams;
import com.smppw.analysis.domain.dto.position.synthesize.AssetAllocationVO;
import com.smppw.analysis.domain.manager.position.AbstractBizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerConstants;
import com.smppw.analysis.domain.manager.position.PositionLoadFactory;
import com.smppw.analysis.domain.dao.PubliclyFundPositionDao;
import com.smppw.analysis.domain.dataobject.FundPositionBaseInfoDO;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.analysis.infrastructure.exception.APIException;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.constants.SecType;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;

/**
 * @author wangzaijun
 * @date 2023/6/7 9:51
 * @description 综合类 业务逻辑处理接口
 */
@Component(BizHandlerConstants.ASSET_ALLOCATION)
public class AssetAllocationBizHandler extends AbstractBizHandler<AssetAllocationParams, List<AssetAllocationVO>> {
    private static final ValueLabelVO RETURN_SALE = new ValueLabelVO(PositionConstants.ASSET_RETURN_SALE, PositionConstants.ASSET_RETURN_SALE_DESC);
    private static final ValueLabelVO MONEY = new ValueLabelVO(PositionConstants.ASSET_MONEY, PositionConstants.ASSET_MONEY_DESC);
//    private static final ValueLabelVO SALE = new ValueLabelVO(PositionConstants.ASSET_SALE, PositionConstants.ASSET_SALE_DESC);
    private static final ValueLabelVO NV = new ValueLabelVO("NV", "净资产");
    private final PubliclyFundPositionDao fundPositionBaseService;

    public AssetAllocationBizHandler(AnalysisProperty property, CacheFactory cacheFactory,
                                     PositionLoadFactory factory, PubliclyFundPositionDao fundPositionBaseService) {
        super(property, cacheFactory, factory);
        this.fundPositionBaseService = fundPositionBaseService;
    }

    @Override
    protected List<AssetAllocationVO> afterLoadData(AssetAllocationParams params, List<FundPositionBaseInfo> baseInfos, List<FundPositionDetail> positionInfos) {
        List<AssetAllocationVO> resultList = ListUtil.list(false);
        String fundType = positionInfos.get(0).getFundType();
        if (SecType.PUBLICLY_OFFERED_FUNDS.equals(fundType)) {
            // 公募基金特殊处理
            List<FundPositionBaseInfoDO> doList = this.fundPositionBaseService.fundPositionBaseInfos(params.getFundId(),
                    params.getStartDate(), params.getEndDate());
            for (FundPositionBaseInfoDO temp : doList) {
                AssetAllocationVO vo = new AssetAllocationVO();
                vo.setDate(DateUtil.formatDate(temp.getReportDate()));
                List<RefMarketValueRatio> assetList = ListUtil.list(false);
                this.handleMvr(assetList, AssetCategoryEnum.buildValueLabelByAsset(AssetCategoryEnum.STOCK), null, temp.getMvOfEquity(), temp.getRinOfEquity());
                this.handleMvr(assetList, AssetCategoryEnum.buildValueLabelByAsset(AssetCategoryEnum.BOND), null, temp.getMvOfFixedIncome(), temp.getRinOfFixedIncome());
                this.handleMvr(assetList, AssetCategoryEnum.buildValueLabelByAsset(AssetCategoryEnum.FUND), null, temp.getMvOfFund(), temp.getRinOfFund());
                this.handleMvr(assetList, AssetCategoryEnum.buildValueLabelByAsset(AssetCategoryEnum.FUTURE), null, temp.getMvOfFixedDeriva(), temp.getRinOfDeriva());
                this.handleMvr(assetList, RETURN_SALE, null, temp.getMvOfReturnSale(), temp.getRinOfReturnSale());
                this.handleMvr(assetList, MONEY, null, temp.getMvOfMoney(), temp.getRinOfMoney());
                // 每个估值日内数据按占比降序
                assetList.sort(Comparator.comparing(RefMarketValueRatio::getRatio).reversed());
                // 降序后把其他和净资产添加到结果中
                this.handleMvr(assetList, OTHER, temp.getAssetNv(), null, null);
                assetList.add(new RefMarketValueRatio(NV, temp.getAssetNv(), BigDecimal.ONE));
                vo.setAsset(assetList);
                resultList.add(vo);
            }
        } else {
            throw new APIException(StrUtil.format("当前基金类型【{} - {}】不支持做大类资产漂移分析！", params.getFundId(), fundType));
        }
        // 按日期升序
        resultList.sort(Comparator.comparing(AssetAllocationVO::getDate));
        return resultList;
    }

    private void handleMvr(List<RefMarketValueRatio> assetList, ValueLabelVO asset, BigDecimal assetNv, BigDecimal mvOf, BigDecimal rinOf) {
        if (OTHER.equals(asset)) {
            BigDecimal totalMv = assetList.stream().map(RefMarketValueRatio::getMarketValue).filter(Objects::nonNull)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalRatio = assetList.stream().map(RefMarketValueRatio::getRatio).filter(Objects::nonNull)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            mvOf = assetNv == null ? null : assetNv.subtract(totalMv);
            rinOf = BigDecimal.ONE.subtract(totalRatio);
        }
        if (mvOf != null || rinOf != null) {
            assetList.add(new RefMarketValueRatio(asset, mvOf, rinOf));
        }
    }
}
