package com.smppw.analysis.domain.manager.position.synthesize;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.NumberUtil;
import com.smppw.analysis.domain.dto.position.FundPositionBaseInfo;
import com.smppw.analysis.domain.dto.position.FundPositionDetail;
import com.smppw.analysis.domain.dto.position.synthesize.PositionListParams;
import com.smppw.analysis.domain.dto.position.synthesize.PositionListVO;
import com.smppw.analysis.domain.manager.position.AbstractBizHandler;
import com.smppw.analysis.domain.manager.position.BizHandlerConstants;
import com.smppw.analysis.domain.manager.position.PositionLoadFactory;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.analysis.infrastructure.exception.APIException;
import com.smppw.constants.SecType;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/6/8 9:47
 * @description 其他特殊的综合组件
 */
@Service(BizHandlerConstants.POSITION_LIST)
public class PositionListBizHandler extends AbstractBizHandler<PositionListParams, List<PositionListVO>> {
    public PositionListBizHandler(AnalysisProperty property, CacheFactory cacheFactory, PositionLoadFactory factory) {
        super(property, cacheFactory, factory);
    }

    @Override
    protected boolean isAllPositionData() {
        return true;
    }

    @Override
    protected void checkParams(PositionListParams params) {
        super.checkParams(params);
        if (CharSequenceUtil.isBlankOrUndefined(params.getDate())) {
            throw new APIException("估值日期不能为空！");
        }
    }

    @Override
    protected List<PositionListVO> afterLoadData(PositionListParams params, List<FundPositionBaseInfo> baseInfos, List<FundPositionDetail> positionInfos) {
        // 公募基金或者私募基金级别大于等于4级，并且数量不为null的
        positionInfos = positionInfos.stream().filter(e -> SecType.PUBLICLY_OFFERED_FUNDS.equals(e.getFundType()) || e.getLevel() >= 4)
                .filter(e -> SecType.PUBLICLY_OFFERED_FUNDS.equals(e.getFundType()) || (e.getSubType() != null && e.getSubType() != 20))
                .filter(e -> e.getAsset() != null).collect(Collectors.toList());
        // 按日期分组，求估值增值之和，得到每个日期的估值增值之和
        Map<String, BigDecimal> collect = baseInfos.stream().filter(e -> e.getIncrement() != null)
                .collect(Collectors.groupingBy(FundPositionBaseInfo::getDate,
                        Collectors.reducing(BigDecimal.ZERO, FundPositionBaseInfo::getIncrement, BigDecimal::add)));
        for (FundPositionDetail detail : positionInfos) {
            BigDecimal increment = detail.getValueChange();
            // 增值比例
            if (increment != null) {
                // 先取基金主体表存储的估值增值
                BigDecimal bigDecimal = baseInfos.stream().filter(e -> detail.getDate().equals(e.getDate())).findFirst()
                        .map(FundPositionBaseInfo::getIncrement).orElse(null);
                // 没有的话取当前日期的成分累计估值增值
                if (bigDecimal == null) {
                    bigDecimal = collect.get(detail.getDate());
                }
                if (bigDecimal != null && bigDecimal.abs().compareTo(BigDecimal.ZERO) > 0) {
                    detail.setValueChangeRatio(increment.divide(bigDecimal, 4, RoundingMode.HALF_UP));
                }
            }
        }
        // 数量变化处理
        this.handleNumberChange(positionInfos);
        List<PositionListVO> resultList = ListUtil.list(true);
        for (FundPositionDetail detail : positionInfos) {
            PositionListVO vo = new PositionListVO();
            vo.setDate(detail.getDate());
            vo.setSuspension(detail.getSuspension());
            vo.setName(detail.getRef().getLabel());
            vo.setType(detail.getAsset().getLabel());
            vo.setCost(detail.getCost() == null ? null : detail.getCost().toPlainString());
            vo.setNum(detail.getNumber() == null ? null : detail.getNumber().toPlainString());
            vo.setNumChange(detail.getNumberChange() == null ? null : detail.getNumberChange().toPlainString());
            vo.setValueChange(detail.getValueChange() == null ? null : detail.getValueChange().toPlainString());
            vo.setValueChangeRatio(detail.getValueChangeRatio() == null ? null : detail.getValueChangeRatio().toPlainString());
            vo.setMarketValue(detail.getMarketValue() == null ? null : detail.getMarketValue().toPlainString());
            if (detail.getRatioNv() != null) {
                vo.setWeight(detail.getRatioNv());
            } else {
                BigDecimal total = baseInfos.stream().filter(e -> detail.getDate().equals(e.getDate())).findFirst()
                        .map(FundPositionBaseInfo::getAssetNv).orElse(null);
                if (total != null && total.abs().compareTo(BigDecimal.ZERO) > 0 && detail.getMarketValue() != null) {
                    vo.setWeight(detail.getMarketValue().divide(total, 10, RoundingMode.HALF_UP));
                }
            }
            resultList.add(vo);
        }
        // 权重降序
        List<PositionListVO> tempList = resultList.stream().filter(e -> e.getWeight() != null)
                .sorted((o1, o2) -> o2.getWeight().compareTo(o1.getWeight())).collect(Collectors.toList());
        // 日期过滤
        tempList = tempList.stream().filter(e -> params.getDate().equals(e.getDate())).collect(Collectors.toList());
        // 大类过滤
        if (CharSequenceUtil.isNotBlank(params.getType()) && !"ALL".equals(params.getType())) {
            tempList = tempList.stream().filter(e -> params.getAssetCategory().getName().equals(e.getType()))
                    .collect(Collectors.toList());
        }
        // 数量过滤
        if (NumberUtil.isInteger(params.getNum()) && !"-1".equals(params.getNum())) {
            tempList = tempList.stream().limit(Long.parseLong(params.getNum())).collect(Collectors.toList());
        }
        if ("-1".equals(params.getNum())) {
            List<PositionListVO> nullList = resultList.stream().filter(e -> e.getWeight() == null).collect(Collectors.toList());
            tempList = CollUtil.unionAll(tempList, nullList);
        }
        return tempList;
    }

    /**
     * 数量变化逻辑处理，本期数量-上一期数量
     *
     * @param dataList /
     */
    protected void handleNumberChange(List<FundPositionDetail> dataList) {
        String minDate = dataList.stream().map(FundPositionDetail::getDate).min(Comparator.naturalOrder()).orElse(null);
        Map<String, BigDecimal> prevMap = MapUtil.newHashMap();
        for (FundPositionDetail temp : dataList) {
            if (temp.getRef() == null) {
                continue;
            }
            String ref = temp.getRef().getValue();
            if (minDate.equals(temp.getDate())) {
                prevMap.put(ref, temp.getNumber());
                continue;
            }
            BigDecimal prev = prevMap.get(ref);
            BigDecimal number = temp.getNumber();
            if (prev != null && number != null) {
                temp.setNumberChange(number.subtract(prev));
            }
            if (number != null) {
                prevMap.put(ref, number);
            }
        }
    }
}
