package com.smppw.analysis.infrastructure.gatewayimpl;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.gateway.CacheFactory;
import com.smppw.analysis.domain.gateway.CacheGateway;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.naming.OperationNotSupportedException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author wangzaijun
 * @date 2023/8/8 9:13
 * @description 简单内存缓存，不支持设置超时时间，限制内存容量为 1 << 5
 */
@Component(CacheFactory.MEMORY)
public class InMemoryCacheGateway implements CacheGateway<Object> {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    private static final Map<String, Object> CACHE_MAP = MapUtil.newConcurrentHashMap();

    @Override
    public Boolean hasKey(String key) {
        return CACHE_MAP.containsKey(key);
    }

    @Override
    public boolean expire(String key, long ttl, TimeUnit timeUnit) {
        this.logger.warn(StrUtil.format("memory cache key:{} no support set ttl!", key));
        return false;
    }

    @Override
    public boolean set(String key, Object value) {
        if (this.addable()) {
            CACHE_MAP.put(key, value);
            return true;
        }
        return false;
    }

    @Override
    public boolean set(String key, Object value, long ttl, TimeUnit timeUnit) {
        if (this.addable()) {
            CACHE_MAP.put(key, value);
            this.logger.warn(StrUtil.format("memory cache key:{} no support set ttl!", key));
            return true;
        }
        return false;
    }

    @Override
    public Object get(String key) {
        return CACHE_MAP.get(key);
    }

    @Override
    public void delete(List<String> keys) {
        CACHE_MAP.keySet().removeIf(keys::contains);
    }

    @Override
    public boolean hset(String key, Map<String, Object> values) {
        if (this.addable()) {
            CACHE_MAP.put(key, values);
            return true;
        }
        return false;
    }

    @Override
    public boolean hset(String key, String item, Object value) {
        if (this.addable()) {
            Map<String, Object> data = MapUtil.<String, Object>builder().put(item, value).build();
            CACHE_MAP.put(key, data);
            return true;
        }
        return false;
    }

    @Override
    public boolean hset(String key, String item, Object value, long ttl, TimeUnit timeUnit) {
        if (this.addable()) {
            Map<String, Object> data = MapUtil.<String, Object>builder().put(item, value).build();
            CACHE_MAP.put(key, data);
            this.logger.warn(StrUtil.format("memory cache key:{} no support set ttl!", key));
            return true;
        }
        return false;
    }

    @Override
    @SuppressWarnings("unchecked")
    public Map<String, Object> hget(String key) {
        return MapUtil.get(CACHE_MAP, key, Map.class);
    }

    @Override
    @SuppressWarnings("unchecked")
    public Object hget(String key, String item) {
        Map<String, Object> data = MapUtil.get(CACHE_MAP, key, Map.class);
        return data.get(item);
    }

    private boolean addable() {
        return CACHE_MAP.size() <= 1 << 5;
    }
}
